@@ -26,8 +26,8 @@
 
 /**
  * SECTION:nc_cluster_mass
- * @title: Cluster Mass Distribution
- * @short_description: FIXME
+ * @title: NcClusterMass
+ * @short_description: Abstract class for cluster mass distributions.
  *
  * FIXME
  */
@@ -38,6 +38,7 @@
 #include "build_cfg.h"
 
 #include "lss/nc_cluster_mass.h"
+#include "math/ncm_serialize.h"
 #include "math/ncm_cfg.h"
 
 G_DEFINE_ABSTRACT_TYPE (NcClusterMass, nc_cluster_mass, NCM_TYPE_MODEL);
@@ -53,10 +54,10 @@ G_DEFINE_ABSTRACT_TYPE (NcClusterMass, nc_cluster_mass, NCM_TYPE_MODEL);
 NcClusterMass *
 nc_cluster_mass_new_from_name (gchar *mass_name)
 {
-  GObject *obj = ncm_cfg_create_from_string (mass_name);
+  GObject *obj = ncm_serialize_global_from_string (mass_name);
   GType mass_type = G_OBJECT_TYPE (obj);
   if (!g_type_is_a (mass_type, NC_TYPE_CLUSTER_MASS))
-	g_error ("nc_cluster_mass_new_from_name: NcClusterMass %s do not descend from %s\n", mass_name, g_type_name (NC_TYPE_CLUSTER_MASS));
+	g_error ("nc_cluster_mass_new_from_name: NcClusterMass %s do not descend from %s.", mass_name, g_type_name (NC_TYPE_CLUSTER_MASS));
   return NC_CLUSTER_MASS (obj);
 }
 
@@ -186,15 +187,16 @@ nc_cluster_mass_intp (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gd
  * @lnM: logarithm base e of the true mass.
  * @lnM_obs: (out): logarithm base e of the observed mass.
  * @lnM_obs_params: (out): observed mass params.
+ * @rng: a #NcmRNG.
  *
  * FIXME
  * 
  * Returns: FIXME
  */
 gboolean
-nc_cluster_mass_resample (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *lnM_obs, gdouble *lnM_obs_params)
+nc_cluster_mass_resample (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *lnM_obs, gdouble *lnM_obs_params, NcmRNG *rng)
 {
-  return NC_CLUSTER_MASS_GET_CLASS (clusterm)->resample (clusterm,model, lnM, z, lnM_obs, lnM_obs_params);
+  return NC_CLUSTER_MASS_GET_CLASS (clusterm)->resample (clusterm,model, lnM, z, lnM_obs, lnM_obs_params, rng);
 }
 
 /**
@@ -259,7 +261,7 @@ _nc_cluster_mass_log_all_models_go (GType model_type, guint n)
  *
  */
 void
-nc_cluster_mass_log_all_models ()
+nc_cluster_mass_log_all_models (void)
 {
   g_message ("# Registred NcClusterMass:%s are:\n", g_type_name (NC_TYPE_CLUSTER_MASS));
   _nc_cluster_mass_log_all_models_go (NC_TYPE_CLUSTER_MASS, 0);
@@ -268,11 +270,13 @@ nc_cluster_mass_log_all_models ()
 static void
 nc_cluster_mass_init (NcClusterMass *nc_cluster_mass)
 {
+  NCM_UNUSED (nc_cluster_mass);
 }
 
 static void
 nc_cluster_mass_finalize (GObject *object)
 {
+  
   /* Chain up : end */
   G_OBJECT_CLASS (nc_cluster_mass_parent_class)->finalize (object);
 }
@@ -286,9 +290,12 @@ nc_cluster_mass_class_init (NcClusterMassClass *klass)
 
   object_class->finalize = nc_cluster_mass_finalize;
 
+  ncm_model_class_add_params (NCM_MODEL_CLASS (klass), 0, 0, 1);
+  
   ncm_mset_model_register_id (NCM_MODEL_CLASS (klass), 
                               "NcClusterMass",
                               "Cluster mass observable relation models.",
                               NULL);
+  ncm_model_class_check_params_info (NCM_MODEL_CLASS (klass));
 }
 
