@@ -25,8 +25,8 @@
 
 /**
  * SECTION:nc_cluster_redshift_nodist
- * @title: Cluster Abundance Redshift No Distribution
- * @short_description: FIXME
+ * @title: NcClusterRedshiftNodist
+ * @short_description: Cluster abundance redshift real redshift distribution.
  *
  * FIXME
  */
@@ -37,6 +37,9 @@
 #include "build_cfg.h"
 
 #include "lss/nc_cluster_redshift_nodist.h"
+#include "math/ncm_cfg.h"
+#include "math/ncm_util.h"
+
 #include <gsl/gsl_math.h>
 
 G_DEFINE_TYPE (NcClusterRedshiftNodist, nc_cluster_redshift_nodist, NC_TYPE_CLUSTER_REDSHIFT);
@@ -46,12 +49,18 @@ enum
   PROP_0,
   PROP_Z_MIN,
   PROP_Z_MAX,
+  PROP_SIZE
 };
 
 static gdouble
 _nc_cluster_redshift_nodist_p (NcClusterRedshift *clusterz, gdouble lnM, gdouble z, gdouble *z_obs, gdouble *z_obs_params)
 {
   g_error ("This object don't implement p.");
+  NCM_UNUSED (clusterz);
+  NCM_UNUSED (lnM);
+  NCM_UNUSED (z);
+  NCM_UNUSED (z_obs);
+  NCM_UNUSED (z_obs_params);
   return GSL_NAN;
 }
 
@@ -59,13 +68,21 @@ static gdouble
 _nc_cluster_redshift_nodist_intp (NcClusterRedshift *clusterz, gdouble lnM, gdouble z)
 {
   g_error ("This object don't implement n_z_lnm.");
+  NCM_UNUSED (clusterz);
+  NCM_UNUSED (lnM);
+  NCM_UNUSED (z);
   return GSL_NAN;
 }
 
 static gboolean
-_nc_cluster_redshift_nodist_resample (NcClusterRedshift *clusterz, gdouble lnM, gdouble z, gdouble *z_obs, gdouble *z_obs_params)
+_nc_cluster_redshift_nodist_resample (NcClusterRedshift *clusterz, gdouble lnM, gdouble z, gdouble *z_obs, gdouble *z_obs_params, NcmRNG *rng)
 {
   NcClusterRedshiftNodist *zn = NC_CLUSTER_REDSHIFT_NODIST (clusterz);
+
+  NCM_UNUSED (lnM);
+  NCM_UNUSED (z);
+  NCM_UNUSED (z_obs_params);
+
   z_obs[0] = z;
   return (z_obs[0] <= zn->z_max) && (z_obs[0] >= zn->z_min);
 }
@@ -74,6 +91,13 @@ static void
 _nc_cluster_redshift_nodist_p_limits (NcClusterRedshift *clusterz, gdouble *z_obs, gdouble *z_obs_params, gdouble *z_lower, gdouble *z_upper)
 {
   g_error ("This object don't implement p_limits.");
+
+  NCM_UNUSED (clusterz);
+  NCM_UNUSED (z_obs);
+  NCM_UNUSED (z_obs_params);
+  NCM_UNUSED (z_lower);
+  NCM_UNUSED (z_upper);
+  
   return;
 }
 
@@ -88,8 +112,8 @@ _nc_cluster_redshift_nodist_n_limits (NcClusterRedshift *clusterz, gdouble *z_lo
   return;
 }
 
-guint _nc_cluster_redshift_nodist_obs_len (NcClusterRedshift *clusterz) { return 1; }
-guint _nc_cluster_redshift_nodist_obs_params_len (NcClusterRedshift *clusterz) { return 0; }
+guint _nc_cluster_redshift_nodist_obs_len (NcClusterRedshift *clusterz) { NCM_UNUSED (clusterz); return 1; }
+guint _nc_cluster_redshift_nodist_obs_params_len (NcClusterRedshift *clusterz) { NCM_UNUSED (clusterz); return 0; }
 
 static void
 _nc_cluster_redshift_nodist_set_property (GObject * object, guint prop_id, const GValue * value, GParamSpec * pspec)
@@ -151,6 +175,7 @@ nc_cluster_redshift_nodist_class_init (NcClusterRedshiftNodistClass *klass)
 {
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
   NcClusterRedshiftClass* parent_class = NC_CLUSTER_REDSHIFT_CLASS (klass);
+  NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
 
   parent_class->P              = &_nc_cluster_redshift_nodist_p;
   parent_class->intP           = &_nc_cluster_redshift_nodist_intp;
@@ -162,10 +187,14 @@ nc_cluster_redshift_nodist_class_init (NcClusterRedshiftNodistClass *klass)
 
   parent_class->impl           = NC_CLUSTER_REDSHIFT_N_LIMTS | NC_CLUSTER_REDSHIFT_RESAMPLE;
 
-  object_class->finalize     =  &nc_cluster_redshift_nodist_finalize;
-  object_class->set_property = &_nc_cluster_redshift_nodist_set_property;
-  object_class->get_property = &_nc_cluster_redshift_nodist_get_property;
+  object_class->finalize    =  &nc_cluster_redshift_nodist_finalize;
+  
+  model_class->set_property = &_nc_cluster_redshift_nodist_set_property;
+  model_class->get_property = &_nc_cluster_redshift_nodist_get_property;
 
+  ncm_model_class_set_name_nick (model_class, "No redshift distribution", "No_distribution");
+  ncm_model_class_add_params (model_class, 0, 0, PROP_SIZE);
+  
   /**
    * NcClusterRedshiftNodist:z_min:
    *
@@ -192,5 +221,6 @@ nc_cluster_redshift_nodist_class_init (NcClusterRedshiftNodistClass *klass)
                                                         0.0, G_MAXDOUBLE, 1.0,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
-
+  /* Check for errors in parameters initialization */
+  ncm_model_class_check_params_info (model_class);
 }
