@@ -25,10 +25,11 @@
 
 /**
  * SECTION:nc_multiplicity_func_st
- * @title: Sheth-Tormen Multiplicity Function
- * @short_description: Dark Matter Halo FIXME
+ * @title: NcMultiplicityFuncST
+ * @short_description: Dark matter halo -- Sheth-Tormen multiplicity function.
  *
  * FIXME
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -86,6 +87,8 @@ _nc_multiplicity_func_st_eval (NcMultiplicityFunc *mulf, NcHICosmo *model, gdoub
   //gdouble f_ST = A * bc1 * (1.0 + pow(x2 * b, -p)) * exp(-(b * x2) / 2.0) * x; // Jenkin's paper
   gdouble f_ST = A * bc1 * (1.0 + pow(x * b, -p)) * exp(-(b2 * x2) / 2.0) * x; // Evrard' s paper
 
+  NCM_UNUSED (model);
+  NCM_UNUSED (z);
  //  printf ("A = %.5g, b=%.5g, p=%.5g, delta_c= %.5g\n", A, b, p, mulf_st->delta_c);
 
   return f_ST;
