@@ -25,8 +25,8 @@
 
 /**
  * SECTION:nc_multiplicity_func_tinker
- * @title: Tinker Multiplicity Function
- * @short_description: Dark Matter Halo FIXME
+ * @title: NcMultiplicityFuncTinker
+ * @short_description: Dark matter halo -- Tinker multiplicity function.
  *
  * FIXME
  */
@@ -85,6 +85,8 @@ _nc_multiplicity_func_tinker_eval (NcMultiplicityFunc *mulf, NcHICosmo *model, g
   const gdouble b = mulf_tinker->b0 * pow(1.0 + z, -alpha);
   const gdouble f_Tinker = A * (pow(sigma/b, -a) + 1.0) * exp(-(mulf_tinker->c) / (sigma * sigma));
 
+  NCM_UNUSED (model);
+  
   //	printf ("%.15g %.15g %.15g %.15g | %.15g %.15g %.15g %.15g\n", mulf_tinker->A0, mulf_tinker->a0, mulf_tinker->b0, mulf_tinker->c,
   //	        A, a, b, mulf_tinker->c);
   return f_Tinker;
