@@ -44,14 +44,6 @@ G_BEGIN_DECLS
 typedef struct _NcTransferFuncCAMBClass NcTransferFuncCAMBClass;
 typedef struct _NcTransferFuncCAMB NcTransferFuncCAMB;
 
-
-
-struct _NcTransferFuncCAMBClass
-{
-  /*< private >*/
-  NcTransferFuncClass parent_class;
-};
-
 struct _NcTransferFuncCAMB
 {
   /*< private >*/
@@ -60,9 +52,15 @@ struct _NcTransferFuncCAMB
   gboolean init;
 };
 
+struct _NcTransferFuncCAMBClass
+{
+  /*< private >*/
+  NcTransferFuncClass parent_class;
+};
+
 GType nc_transfer_func_camb_get_type (void) G_GNUC_CONST;
 
-NcTransferFunc *nc_transfer_func_camb_new ();
+NcTransferFunc *nc_transfer_func_camb_new (void);
 
 extern gchar *camb_filename;
 
