@@ -34,6 +34,9 @@
 #include <gsl/gsl_matrix.h>
 #include <gsl/gsl_spline.h>
 #include <gsl/gsl_rng.h>
+#ifndef NUMCOSMO_GIR_SCAN
+#include <gmp.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 #ifdef NUMCOSMO_HAVE_SQLITE3
 #include <sqlite3.h>
 #endif
@@ -41,44 +44,41 @@
 
 G_BEGIN_DECLS
 
-#ifndef __STDC_IEC_559__
-#error "Error: "PACKAGE_NAME" requires IEEE 754 floating point."
-#endif /* __STDC_IEC_559__ */
-
 void ncm_cfg_init (void);
 void ncm_cfg_enable_gsl_err_handler (void);
 void ncm_cfg_register_obj (GType obj);
 gchar *ncm_cfg_get_fullpath (const gchar *filename, ...);
-void ncm_cfg_keyfile_to_arg (GKeyFile *kfile, gchar *group_name, GOptionEntry *entries, gchar **argv, gint *argc);
-void ncm_cfg_entries_to_keyfile (GKeyFile *kfile, gchar *group_name, GOptionEntry *entries);
+void ncm_cfg_keyfile_to_arg (GKeyFile *kfile, const gchar *group_name, GOptionEntry *entries, gchar **argv, gint *argc);
+void ncm_cfg_entries_to_keyfile (GKeyFile *kfile, const gchar *group_name, GOptionEntry *entries);
 gchar *ncm_cfg_string_to_comment (const gchar *str);
 const GEnumValue *ncm_cfg_get_enum_by_id_name_nick (GType enum_type, const gchar *id_name_nick);
 const GEnumValue *ncm_cfg_enum_get_value (GType enum_type, guint n);
-void ncm_cfg_enum_print_all (GType enum_type, gchar *header);
+void ncm_cfg_enum_print_all (GType enum_type, const gchar *header);
 
-gboolean ncm_cfg_load_fftw_wisdom (gchar *filename, ...);
-gboolean ncm_cfg_save_fftw_wisdom (gchar *filename, ...);
-gboolean ncm_cfg_exists (gchar *filename, ...);
+gboolean ncm_cfg_load_fftw_wisdom (const gchar *filename, ...);
+gboolean ncm_cfg_save_fftw_wisdom (const gchar *filename, ...);
+gboolean ncm_cfg_exists (const gchar *filename, ...);
 
 void ncm_cfg_set_logfile (gchar *filename);
 void ncm_cfg_logfile (gboolean on);
 void ncm_cfg_logfile_flush (gboolean on);
+void ncm_cfg_logfile_flush_now (void);
 
-void ncm_message (gchar *msg, ...);
+void ncm_message (const gchar *msg, ...);
 gchar *ncm_string_ww (const gchar *msg, const gchar *first, const gchar *rest, guint ncols);
 void ncm_message_ww (const gchar *msg, const gchar *first, const gchar *rest, guint ncols);
 void ncm_cfg_msg_sepa (void);
 
-FILE *ncm_cfg_fopen (gchar *filename, gchar *mode, ...);
-FILE *ncm_cfg_vfopen (gchar *filename, gchar *mode, va_list ap);
+FILE *ncm_cfg_fopen (const gchar *filename, const gchar *mode, ...);
+FILE *ncm_cfg_vfopen (const gchar *filename, const gchar *mode, va_list ap);
 
-gboolean ncm_cfg_load_spline (gchar *filename, const gsl_interp_type *stype, NcmSpline **s, ...);
-gboolean ncm_cfg_save_spline (gchar *filename, NcmSpline *s, ...);
+gboolean ncm_cfg_load_spline (const gchar *filename, const gsl_interp_type *stype, NcmSpline **s, ...);
+gboolean ncm_cfg_save_spline (const gchar *filename, NcmSpline *s, ...);
 
-gboolean ncm_cfg_load_vector (gchar *filename, gsl_vector *v, ...);
-gboolean ncm_cfg_save_vector (gchar *filename, gsl_vector *v, ...);
-gboolean ncm_cfg_load_matrix (gchar *filename, gsl_matrix *M, ...);
-gboolean ncm_cfg_save_matrix (gchar *filename, gsl_matrix *M, ...);
+gboolean ncm_cfg_load_vector (const gchar *filename, gsl_vector *v, ...);
+gboolean ncm_cfg_save_vector (const gchar *filename, gsl_vector *v, ...);
+gboolean ncm_cfg_load_matrix (const gchar *filename, gsl_matrix *M, ...);
+gboolean ncm_cfg_save_matrix (const gchar *filename, gsl_matrix *M, ...);
 
 #define LOAD_SAVE_VECTOR_MATRIX_DEF(typen) \
 gboolean ncm_cfg_load_vector_##typen (gchar *filename, gsl_vector_##typen *v, ...); \
@@ -90,6 +90,8 @@ LOAD_SAVE_VECTOR_MATRIX_DEF(int)
 LOAD_SAVE_VECTOR_MATRIX_DEF(float)
 LOAD_SAVE_VECTOR_MATRIX_DEF(complex)
 
+gchar *ncm_cfg_get_data_filename (const gchar *filename, gboolean must_exist);
+
 #ifdef NUMCOSMO_HAVE_SQLITE3
 sqlite3 *ncm_cfg_get_default_sqlite3 (void);
 #endif
@@ -106,13 +108,13 @@ GArray *ncm_cfg_variant_to_array (GVariant *var, gsize esize);
 void ncm_cfg_array_set_variant (GArray *a, GVariant *var);
 GVariant *ncm_cfg_array_to_variant (GArray *a, const GVariantType *etype);
 
+void ncm_cfg_set_fftw_default_flag (guint flag);
+
+extern guint fftw_default_flags;
+
 /* Macros */
 
-#ifdef SUNDIALS_USES_LONG_INT
-#define _NCM_SUNDIALS_INT_TYPE glong
-#else
-#define _NCM_SUNDIALS_INT_TYPE gint
-#endif
+#define NCM_CFG_DATA_DIR_ENV "NUMCOSMO_DATA_DIR"
 
 #if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32)
 #define _NCM_MUTEX_LOCK(l) g_static_mutex_lock (l)
@@ -151,7 +153,14 @@ GVariant *ncm_cfg_array_to_variant (GArray *a, const GVariantType *etype);
 
 /* Workaround on g_clear_pointer */
 #if ((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 34))
-#define g_clear_pointer(ptr,freefunc) G_STMT_START if (*(ptr) != NULL) { *(freefunc) (*(ptr)); *(ptr) = NULL; } G_STMT_END
+#define g_clear_pointer(ptr,freefunc) \
+G_STMT_START { \
+  if (*(ptr) != NULL) \
+  { \
+    freefunc (*(ptr)); \
+    *(ptr) = NULL; \
+  } \
+} G_STMT_END
 #endif /* Glib version < 2.34 */
 
 /* Workaround on g_clear_object. */
