@@ -25,8 +25,8 @@
 
 /**
  * SECTION:ncm_data_gauss
- * @title: Gaussian Data - InvCov
- * @short_description: Gaussian data object, inverse covariance
+ * @title: NcmDataGauss
+ * @short_description: Gaussian data -- inverse covariance provided.
  *
  * Gaussian distribution which uses the inverse covariance matrix as input.
  * 
@@ -210,14 +210,17 @@ static void
 _ncm_data_gauss_prepare_LLT (NcmData *data)
 {
   NcmDataGauss *gauss = NCM_DATA_GAUSS (data);
+  gint ret;
 
   if (gauss->LLT == NULL)
     gauss->LLT = ncm_matrix_dup (gauss->inv_cov);
   else
     ncm_matrix_memcpy (gauss->LLT, gauss->inv_cov);
 
-  ncm_matrix_cholesky_decomp (gauss->LLT);
-
+  ret = ncm_matrix_cholesky_decomp (gauss->LLT, 'U');
+  if (ret != 0)
+    g_error ("_ncm_data_gauss_prepare_LLT[ncm_matrix_cholesky_decomp]: %d.", ret);
+  
   gauss->prepared_LLT = TRUE;
 }
 
@@ -243,8 +246,9 @@ _ncm_data_gauss_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng)
     ncm_vector_set (gauss->v, i, u_i);
   }
   ncm_rng_unlock (rng);
-  
-  ret = gsl_blas_dtrsv (CblasLower, CblasTrans, CblasNonUnit, 
+
+  /* CblasLower, CblasTrans => CblasUpper, CblasNoTrans */
+  ret = gsl_blas_dtrsv (CblasUpper, CblasNoTrans, CblasNonUnit, 
                         ncm_matrix_gsl (gauss->LLT), ncm_vector_gsl (gauss->v));
   NCM_TEST_GSL_RESULT ("_ncm_data_gauss_resample", ret);
   
@@ -289,7 +293,8 @@ _ncm_data_gauss_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
     if (inv_cov_update || !gauss->prepared_LLT)
       _ncm_data_gauss_prepare_LLT (data);
 
-    ret = gsl_blas_dtrmv (CblasLower, CblasTrans, CblasNonUnit, 
+    /* CblasLower, CblasTrans => CblasUpper, CblasNoTrans */
+    ret = gsl_blas_dtrmv (CblasUpper, CblasNoTrans, CblasNonUnit, 
                           ncm_matrix_gsl (gauss->LLT), ncm_vector_gsl (gauss->v));
     NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_resample", ret);
     
@@ -322,7 +327,8 @@ _ncm_data_gauss_leastsquares_f (NcmData *data, NcmMSet *mset, NcmVector *v)
   if (inv_cov_update || !gauss->prepared_LLT)
     _ncm_data_gauss_prepare_LLT (data);
 
-  ret = gsl_blas_dtrmv (CblasLower, CblasTrans, CblasNonUnit, 
+  /* CblasLower, CblasTrans => CblasUpper, CblasNoTrans */
+  ret = gsl_blas_dtrmv (CblasUpper, CblasNoTrans, CblasNonUnit, 
                         ncm_matrix_gsl (gauss->LLT), ncm_vector_gsl (v));
   NCM_TEST_GSL_RESULT("_ncm_data_gauss_leastsquares_f", ret);
 }
