@@ -133,17 +133,6 @@ typedef enum _NcmFitRunMsgs
   NCM_FIT_RUN_MSGS_FULL,
 } NcmFitRunMsgs;
 
-struct _NcmFitClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-  NcmFit *(*copy_new) (NcmFit *fit, NcmLikelihood *lh, NcmMSet *mset, NcmFitGradType gtype);
-  void (*reset) (NcmFit *fit);
-  gboolean (*run) (NcmFit *fit, NcmFitRunMsgs mtype);
-  const gchar *(*get_desc) (NcmFit *fit);
-  gboolean is_least_squares;
-};
-
 struct _NcmFit
 {
   /*< private >*/
@@ -155,12 +144,24 @@ struct _NcmFit
   NcmFitGrad grad;
   guint maxiter;
   gdouble m2lnL_reltol;
+  gdouble m2lnL_abstol;
   gdouble params_reltol;
   GTimer *timer;
   GPtrArray *equality_constraints;
   GPtrArray *inequality_constraints;
 };
 
+struct _NcmFitClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+  NcmFit *(*copy_new) (NcmFit *fit, NcmLikelihood *lh, NcmMSet *mset, NcmFitGradType gtype);
+  void (*reset) (NcmFit *fit);
+  gboolean (*run) (NcmFit *fit, NcmFitRunMsgs mtype);
+  const gchar *(*get_desc) (NcmFit *fit);
+  gboolean is_least_squares;
+};
+
 struct _NcmFitConstraint
 {
   /*< private >*/
@@ -179,13 +180,21 @@ void ncm_fit_constraint_free (NcmFitConstraint *fitc);
 NcmFit *ncm_fit_new (NcmFitType ftype, gchar *algo_name, NcmLikelihood *lh, NcmMSet *mset, NcmFitGradType gtype);
 NcmFit *ncm_fit_ref (NcmFit *fit);
 NcmFit *ncm_fit_copy_new (NcmFit *fit, NcmLikelihood *lh, NcmMSet *mset, NcmFitGradType gtype);
+NcmFit *ncm_fit_dup (NcmFit *fit, NcmSerialize *ser);
 void ncm_fit_free (NcmFit *fit);
 void ncm_fit_clear (NcmFit **fit);
 
 void ncm_fit_set_grad_type (NcmFit *fit, NcmFitGradType gtype);
-void ncm_fit_ls_set_state (NcmFit *fit, gdouble prec, NcmVector *x, NcmVector *f, NcmMatrix *J);
+
 void ncm_fit_set_maxiter (NcmFit *fit, guint maxiter);
 guint ncm_fit_get_maxiter (NcmFit *fit);
+void ncm_fit_set_m2lnL_reltol (NcmFit *fit, gdouble tol);
+gdouble ncm_fit_get_m2lnL_reltol (NcmFit *fit);
+void ncm_fit_set_m2lnL_abstol (NcmFit *fit, gdouble tol);
+gdouble ncm_fit_get_m2lnL_abstol (NcmFit *fit);
+void ncm_fit_set_params_reltol (NcmFit *fit, gdouble tol);
+gdouble ncm_fit_get_params_reltol (NcmFit *fit);
+void ncm_fit_set_params (NcmFit *fit, NcmVector *params);
 
 void ncm_fit_add_equality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot);
 void ncm_fit_add_inequality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot);
@@ -218,6 +227,7 @@ void ncm_fit_m2lnL_grad_an (NcmFit *fit, NcmVector *df);
 void ncm_fit_m2lnL_grad_nd_fo (NcmFit *fit, NcmVector *grad);
 void ncm_fit_m2lnL_grad_nd_ce (NcmFit *fit, NcmVector *grad);
 void ncm_fit_m2lnL_grad_nd_ac (NcmFit *fit, NcmVector *grad);
+void ncm_fit_m2lnL_hessian_nd_ce (NcmFit *fit, NcmMatrix *hessian);
 
 void ncm_fit_m2lnL_val_grad (NcmFit *fit, gdouble *result, NcmVector *df);
 void ncm_fit_m2lnL_val_grad_an (NcmFit *fit, gdouble *result, NcmVector *df);
@@ -235,9 +245,10 @@ void ncm_fit_ls_f_J_an (NcmFit *fit, NcmVector *f, NcmMatrix *J);
 void ncm_fit_ls_f_J_nd_fo (NcmFit *fit, NcmVector *f, NcmMatrix *J);
 void ncm_fit_ls_f_J_nd_ce (NcmFit *fit, NcmVector *f, NcmMatrix *J);
 
-void ncm_fit_numdiff_m2lnL_hessian (NcmFit *fit, NcmMatrix *H);
+void ncm_fit_numdiff_m2lnL_hessian (NcmFit *fit, NcmMatrix *H, gdouble reltol);
 void ncm_fit_numdiff_m2lnL_covar (NcmFit *fit);
 void ncm_fit_ls_covar (NcmFit *fit);
+gdouble ncm_fit_numdiff_m2lnL_lndet_covar (NcmFit *fit);
 
 gdouble ncm_fit_covar_var (NcmFit *fit, NcmModelID mid, guint pid);
 gdouble ncm_fit_covar_sd (NcmFit *fit, NcmModelID mid, guint pid);
@@ -249,6 +260,7 @@ gdouble ncm_fit_covar_fparam_sd (NcmFit *fit, guint fpi);
 gdouble ncm_fit_covar_fparam_cov (NcmFit *fit, guint fpi1, guint fpi2);
 gdouble ncm_fit_covar_fparam_cor (NcmFit *fit, guint fpi1, guint fpi2);
 
+gdouble ncm_fit_residual_ks_test (NcmFit *fit, gdouble *o_mean, gdouble *o_sd, gdouble *o_skew, gdouble *o_kurtosis, gdouble *o_max);
 void ncm_fit_lr_test_range (NcmFit *fit, NcmModelID mid, guint pid, gdouble start, gdouble stop, gdouble step);
 void ncm_fit_dprob (NcmFit *fit, NcmModelID mid, guint pid, gdouble a, gdouble b, gdouble step, gdouble norm);
 gdouble ncm_fit_lr_test (NcmFit *fit, NcmModelID mid, guint pid, gdouble val, gint dof);
@@ -262,11 +274,11 @@ gdouble ncm_fit_type_constrain_error (NcmFit *fit, gdouble p, gint nu, gdouble d
 void ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble *x, gboolean pretty_print, gdouble *f, gdouble *sigma_f);
 gdouble ncm_fit_function_cov (NcmFit *fit, NcmMSetFunc *func1, gdouble z1, NcmMSetFunc *func2, gdouble z2, gboolean pretty_print);
 
-#define NCM_FIT_NUMDIFF_SCALE (1.0e-4)
 #define NCM_FIT_NPARAM(fit) ((fit)->pt->nfree)
+#define NCM_FIT_DEFAULT_M2LNL_RELTOL (1e-8)
 #define NCM_FIT_DEFAULT_M2LNL_ABSTOL (0.0)
-#define NCM_FIT_DEFAULT_M2LNL_RELTOL (1e-13)
-#define NCM_FIT_MAXITER 10000
+#define NCM_FIT_DEFAULT_PARAMS_RELTOL (1e-8)
+#define NCM_FIT_DEFAULT_MAXITER 10000
 
 G_END_DECLS
 
