@@ -25,10 +25,11 @@
 
 /**
  * SECTION:ncm_fit_gsl_ls
- * @title: Least Squares -- GSL
- * @short_description: Interface for GSL least squares algorithms
+ * @title: NcmFitGSLLS
+ * @short_description: Best-fit finder -- GSL least squares algorithms.
  *
  * FIXME
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -149,6 +150,8 @@ _ncm_fit_gsl_ls_run (NcmFit *fit, NcmFitRunMsgs mtype)
 
   if (ncm_fit_has_equality_constraints (fit) || ncm_fit_has_inequality_constraints (fit))
     g_error ("_ncm_fit_gsl_ls_run: GSL algorithms do not support constraints.");
+
+  g_assert (fit->fstate->fparam_len != 0);
   
   ncm_mset_fparams_get_vector (fit->mset, fit->fstate->fparams);
   gsl_multifit_fdfsolver_set (fit_gsl_ls->ls, &fit_gsl_ls->f, ncm_vector_gsl (fit->fstate->fparams));
@@ -257,7 +260,9 @@ _ncm_fit_gsl_ls_get_desc (NcmFit *fit)
   if (desc == NULL)
   {
     NcmFitGSLLS *fit_gsl_ls = NCM_FIT_GSL_LS (fit);
-    desc = g_strdup_printf ("GSL Least Squares:%s", gsl_multifit_fdfsolver_name (fit_gsl_ls->ls));
+    desc = g_strdup_printf ("GSL Least Squares:%s", 
+                            fit_gsl_ls->ls != NULL ? gsl_multifit_fdfsolver_name (fit_gsl_ls->ls) : "not-set"
+                            );
   }
   return desc;
 }
