@@ -25,8 +25,8 @@
 
 /**
  * SECTION:ncm_fit_mc
- * @title: Monte Carlo Analysis
- * @short_description: Object implementing Monte Carlo analysis
+ * @title: NcmFitMC
+ * @short_description: Monte Carlo analysis.
  *
  * FIXME
  * 
@@ -52,6 +52,7 @@ enum
   PROP_FIDUC,
   PROP_MTYPE,
   PROP_NTHREADS,
+  PROP_KEEP_ORDER,
   PROP_DATA_FILE,
 };
 
@@ -62,14 +63,15 @@ ncm_fit_mc_init (NcmFitMC *mc)
 {
   mc->fit             = NULL;
   mc->fiduc           = NULL;
+  mc->mcat            = NULL;
   mc->mtype           = NCM_FIT_RUN_MSGS_NONE;
   mc->rtype           = NCM_FIT_MC_RESAMPLE_BOOTSTRAP_LEN;
-  mc->rng             = NULL;
   mc->bf              = NULL;
   mc->nt              = ncm_timer_new ();
   mc->ser             = ncm_serialize_new (NCM_SERIALIZE_OPT_CLEAN_DUP);
   mc->nthreads        = 0;
   mc->n               = 0;
+  mc->keep_order      = FALSE;
   mc->mp              = NULL;
   mc->cur_sample_id   = -1; /* Represents that no samples were calculated yet. */
   mc->write_index     = 0;
@@ -115,6 +117,9 @@ ncm_fit_mc_set_property (GObject *object, guint prop_id, const GValue *value, GP
     case PROP_NTHREADS:
       ncm_fit_mc_set_nthreads (mc, g_value_get_uint (value));
       break;
+    case PROP_KEEP_ORDER:
+      ncm_fit_mc_keep_order (mc, g_value_get_boolean (value));
+      break;
     case PROP_DATA_FILE:
       ncm_fit_mc_set_data_file (mc, g_value_get_string (value));
       break;    
@@ -147,8 +152,11 @@ ncm_fit_mc_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
     case PROP_NTHREADS:
       g_value_set_uint (value, mc->nthreads);
       break;
+    case PROP_KEEP_ORDER:
+      g_value_set_boolean (value, mc->keep_order);
+      break;
     case PROP_DATA_FILE:
-      g_value_set_string (value, ncm_fit_catalog_peek_filename (mc->fcat));
+      g_value_set_string (value, ncm_mset_catalog_peek_filename (mc->mcat));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -166,7 +174,7 @@ ncm_fit_mc_dispose (GObject *object)
   ncm_vector_clear (&mc->bf);
   ncm_timer_clear (&mc->nt);
   ncm_serialize_clear (&mc->ser);
-  ncm_fit_catalog_clear (&mc->fcat);
+  ncm_mset_catalog_clear (&mc->mcat);
 
   if (mc->mp != NULL)
   {
@@ -218,7 +226,7 @@ ncm_fit_mc_class_init (NcmFitMCClass *klass)
                                    PROP_RTYPE,
                                    g_param_spec_enum ("rtype",
                                                       NULL,
-                                                      "Montecarlo run type",
+                                                      "Monte Carlo run type",
                                                       NCM_TYPE_FIT_MC_RESAMPLE_TYPE, NCM_FIT_MC_RESAMPLE_FROM_MODEL,
                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
@@ -236,6 +244,13 @@ ncm_fit_mc_class_init (NcmFitMCClass *klass)
                                                       "Run messages type",
                                                       NCM_TYPE_FIT_RUN_MSGS, NCM_FIT_RUN_MSGS_SIMPLE,
                                                       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_KEEP_ORDER,
+                                   g_param_spec_boolean ("keep-order",
+                                                         NULL,
+                                                         "Whether keep the catalog in order of sampling",
+                                                         TRUE,
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_NTHREADS,
                                    g_param_spec_uint ("nthreads",
@@ -250,7 +265,7 @@ _ncm_fit_mc_set_fit_obj (NcmFitMC *mc, NcmFit *fit)
 {
   g_assert (mc->fit == NULL);
   mc->fit = ncm_fit_ref (fit);
-  mc->fcat = ncm_fit_catalog_new (fit);
+  mc->mcat = ncm_mset_catalog_new (fit->mset, 1, 1, FALSE, NCM_MSET_CATALOG_M2LNL_COLNAME);
 }
 
 /**
@@ -266,16 +281,17 @@ _ncm_fit_mc_set_fit_obj (NcmFitMC *mc, NcmFit *fit)
 NcmFitMC *
 ncm_fit_mc_new (NcmFit *fit, NcmFitMCResampleType rtype, NcmFitRunMsgs mtype)
 {
-  return g_object_new (NCM_TYPE_FIT_MC, 
-                       "fit", fit,
-                       "rtype", rtype,
-                       "mtype", mtype,
-                       NULL);
+  NcmFitMC * mc = g_object_new (NCM_TYPE_FIT_MC, 
+                                "fit", fit,
+                                "rtype", rtype,
+                                "mtype", mtype,
+                                NULL);
+  return mc;
 }
 
 /**
  * ncm_fit_mc_free:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  *
  * FIXME
  *
@@ -288,7 +304,7 @@ ncm_fit_mc_free (NcmFitMC *mc)
 
 /**
  * ncm_fit_mc_clear:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  *
  * FIXME
  *
@@ -303,7 +319,7 @@ static void ncm_fit_mc_intern_skip (NcmFitMC *mc, guint n);
 
 /**
  * ncm_fit_mc_set_data_file:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * @filename: a filename.
  *
  * FIXME
@@ -312,64 +328,33 @@ static void ncm_fit_mc_intern_skip (NcmFitMC *mc, guint n);
 void 
 ncm_fit_mc_set_data_file (NcmFitMC *mc, const gchar *filename)
 {
-  const gchar *cur_filename = ncm_fit_catalog_peek_filename (mc->fcat); 
+  const gchar *cur_filename = ncm_mset_catalog_peek_filename (mc->mcat); 
   if (mc->started && cur_filename != NULL)
     g_error ("ncm_fit_mc_set_data_file: Cannot change data file during a run, call ncm_fit_mc_end_run() first.");    
 
   if (cur_filename != NULL && strcmp (cur_filename, filename) == 0)
     return;
 
-  ncm_fit_catalog_set_file (mc->fcat, filename);
+  ncm_mset_catalog_set_file (mc->mcat, filename);
 
   if (mc->started)
   {
-    gulong seed;
-    gchar *algo = NULL;
-    if (mc->fcat->cur_id > mc->cur_sample_id)
+    if (mc->mcat->cur_id > mc->cur_sample_id)
     {
-      ncm_fit_mc_intern_skip (mc, mc->fcat->cur_id - mc->cur_sample_id);
-      g_assert_cmpint (mc->cur_sample_id, ==, mc->fcat->cur_id);
+      ncm_fit_mc_intern_skip (mc, mc->mcat->cur_id - mc->cur_sample_id);
+      g_assert_cmpint (mc->cur_sample_id, ==, mc->mcat->cur_id);
     }
-    else if (mc->fcat->cur_id < mc->cur_sample_id)
+    else if (mc->mcat->cur_id < mc->cur_sample_id)
+    {
       g_error ("ncm_fit_mc_set_data_file: Unknown error cur_id < cur_sample_id [%d < %d].", 
-               mc->fcat->cur_id, mc->cur_sample_id);
-
-    if (ncm_fit_catalog_get_prng (mc->fcat, &algo, &seed))
-    {
-      g_assert_cmpstr (algo, ==, ncm_rng_get_algo (mc->rng));
-      g_assert_cmpuint (seed, ==, ncm_rng_get_seed (mc->rng));
-      g_free (algo);
+               mc->mcat->cur_id, mc->cur_sample_id);
     }
-    else
-      ncm_fit_catalog_set_prng (mc->fcat, mc->rng);
-  }
-  else
-  {
-    gulong seed;
-    gchar *algo = NULL;
-    if (ncm_fit_catalog_get_prng (mc->fcat, &algo, &seed))
-    {
-      if (mc->rng == NULL)
-      {
-        NcmRNG *rng = ncm_rng_seeded_new (algo, seed);
-        ncm_fit_mc_set_rng (mc, rng);
-        ncm_rng_free (rng);
-      }
-      else
-      {
-        g_assert_cmpstr (algo, ==, ncm_rng_get_algo (mc->rng));
-        g_assert_cmpuint (seed, ==, ncm_rng_get_seed (mc->rng));
-      }
-      g_free (algo);
-    }
-    else if (mc->rng != NULL)
-      ncm_fit_catalog_set_prng (mc->fcat, mc->rng);
   }
 }
 
 /**
  * ncm_fit_mc_set_mtype:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * @mtype: FIXME
  *
  * FIXME
@@ -386,14 +371,14 @@ static void _ncm_fit_mc_resample_bstrap (NcmDataset *dset, NcmMSet *mset, NcmRNG
 static gint
 _ncm_fit_mc_resample (NcmFitMC *mc, NcmFit *fit)
 {
-  mc->resample (fit->lh->dset, mc->fiduc, mc->rng);
+  mc->resample (fit->lh->dset, mc->fiduc, mc->mcat->rng);
   mc->cur_sample_id++;
   return mc->cur_sample_id;
 }
 
 /**
  * ncm_fit_mc_set_rtype:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * @rtype: FIXME
  *
  * FIXME
@@ -409,7 +394,7 @@ ncm_fit_mc_set_rtype (NcmFitMC *mc, NcmFitMCResampleType rtype)
 
   mc->rtype = rtype;
 
-  ncm_fit_catalog_set_run_type (mc->fcat, eval->value_nick);
+  ncm_mset_catalog_set_run_type (mc->mcat, eval->value_nick);
   
   switch (rtype)
   {
@@ -433,7 +418,7 @@ ncm_fit_mc_set_rtype (NcmFitMC *mc, NcmFitMCResampleType rtype)
 
 /**
  * ncm_fit_mc_set_nthreads:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * @nthreads: FIXME
  *
  * FIXME
@@ -445,9 +430,23 @@ ncm_fit_mc_set_nthreads (NcmFitMC *mc, guint nthreads)
   mc->nthreads = nthreads;
 }
 
+/**
+ * ncm_fit_mc_keep_order:
+ * @mc: a #NcmFitMC
+ * @keep_order: FIXME
+ *
+ * FIXME
+ *
+ */
+void 
+ncm_fit_mc_keep_order (NcmFitMC *mc, gboolean keep_order)
+{
+  mc->keep_order = keep_order;
+}
+
 /**
  * ncm_fit_mc_set_fiducial:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * @fiduc: FIXME
  *
  * FIXME
@@ -462,7 +461,7 @@ ncm_fit_mc_set_fiducial (NcmFitMC *mc, NcmMSet *fiduc)
   if (fiduc == NULL || fiduc == mc->fit->mset)
   {
     mc->fiduc = ncm_mset_dup (mc->fit->mset, mc->ser);
-    ncm_serialize_reset (mc->ser);
+    ncm_serialize_clear_instances (mc->ser);
   }
   else
   {
@@ -473,7 +472,7 @@ ncm_fit_mc_set_fiducial (NcmFitMC *mc, NcmMSet *fiduc)
 
 /**
  * ncm_fit_mc_set_rng:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * @rng: FIXME
  *
  * FIXME
@@ -481,28 +480,11 @@ ncm_fit_mc_set_fiducial (NcmFitMC *mc, NcmMSet *fiduc)
  */
 void
 ncm_fit_mc_set_rng (NcmFitMC *mc, NcmRNG *rng)
-{
-  gchar *algo = NULL;
-  gulong seed = 0;
-  
+{  
   if (mc->started)
-    g_error ("ncm_fit_mc_set_rng: Cannot change the PRNG object during a run, call ncm_fit_mc_end_run() first.");
+    g_error ("ncm_fit_mc_set_rng: Cannot change the RNG object during a run, call ncm_fit_mc_end_run() first.");
 
-  ncm_rng_clear (&mc->rng);
-  mc->rng = ncm_rng_ref (rng);
-
-  if (ncm_fit_catalog_peek_filename (mc->fcat) != NULL)
-  {
-    if (ncm_fit_catalog_get_prng (mc->fcat, &algo, &seed))
-    {
-      if (strcmp (algo, ncm_rng_get_algo (rng)) != 0 || seed != ncm_rng_get_seed (rng))
-        g_error ("ncm_fit_mc_set_rng: catalog has PRNG with algorithm: `%s' and seed: %lu, and the montecarlo object has algorithm: `%s' and seed: %lu.",
-                 algo, seed, ncm_rng_get_algo (rng), ncm_rng_get_seed (rng));
-      g_free (algo);
-    }
-    else
-      ncm_fit_catalog_set_prng (mc->fcat, rng);
-  }
+  ncm_mset_catalog_set_rng (mc->mcat, rng);
 }
 
 void
@@ -511,7 +493,7 @@ _ncm_fit_mc_update (NcmFitMC *mc, NcmFit *fit)
   const guint part = 5;
   const guint step = (mc->n / part) == 0 ? 1 : (mc->n / part);
 
-  ncm_fit_catalog_add_from_fit (mc->fcat, fit);
+  ncm_mset_catalog_add_from_mset (mc->mcat, fit->mset, ncm_fit_state_get_m2lnL_curval (fit->fstate));
   ncm_timer_task_increment (mc->nt);
 
   switch (mc->mtype)
@@ -521,10 +503,13 @@ _ncm_fit_mc_update (NcmFitMC *mc, NcmFit *fit)
     case NCM_FIT_RUN_MSGS_SIMPLE:
     {
       guint stepi = mc->nt->task_pos % step;
-      if ((stepi == 0) || (mc->nt->task_pos == mc->nt->task_len))
+      gboolean log_timeout = FALSE;
+      if ((mc->nt->pos_time - mc->nt->last_log_time) > 60.0)
+        log_timeout = TRUE;
+      if (log_timeout || (stepi == 0) || (mc->nt->task_pos == mc->nt->task_len))
       {
         /* guint acc = stepi == 0 ? step : stepi; */
-        ncm_fit_catalog_log_current_stats (mc->fcat);
+        ncm_mset_catalog_log_current_stats (mc->mcat);
         /* ncm_timer_task_accumulate (mc->nt, acc); */
         ncm_timer_task_log_elapsed (mc->nt);
         ncm_timer_task_log_mean_time (mc->nt);
@@ -538,7 +523,7 @@ _ncm_fit_mc_update (NcmFitMC *mc, NcmFit *fit)
       fit->mtype = mc->mtype;
       ncm_fit_log_start (fit);
       ncm_fit_log_end (fit);
-      ncm_fit_catalog_log_current_stats (mc->fcat);
+      ncm_mset_catalog_log_current_stats (mc->mcat);
       /* ncm_timer_task_increment (mc->nt); */
       ncm_timer_task_log_elapsed (mc->nt);
       ncm_timer_task_log_mean_time (mc->nt);
@@ -547,11 +532,11 @@ _ncm_fit_mc_update (NcmFitMC *mc, NcmFit *fit)
       break;      
   }
 
-  if ((mc->fcat->fmode != NCM_FIT_MC_MIN_FLUSH_INTERVAL) &&
+  if ((mc->mcat->fmode != NCM_FIT_MC_MIN_FLUSH_INTERVAL) &&
       (ncm_timer_task_mean_time (mc->nt) < NCM_FIT_MC_MIN_FLUSH_INTERVAL))
   {
-    ncm_fit_catalog_set_flush_mode (mc->fcat, NCM_FIT_CATALOG_FLUSH_TIMED);
-    ncm_fit_catalog_set_flush_interval (mc->fcat, NCM_FIT_MC_MIN_FLUSH_INTERVAL);
+    ncm_mset_catalog_set_flush_mode (mc->mcat, NCM_MSET_CATALOG_FLUSH_TIMED);
+    ncm_mset_catalog_set_flush_interval (mc->mcat, NCM_FIT_MC_MIN_FLUSH_INTERVAL);
   }
 }
 
@@ -585,7 +570,7 @@ ncm_fit_mc_start_run (NcmFitMC *mc)
     default:
     case NCM_FIT_RUN_MSGS_FULL:
       ncm_cfg_msg_sepa ();
-      g_message ("# NcmFitMC: Starting Montecarlo...\n");
+      g_message ("# NcmFitMC: Starting Monte Carlo...\n");
       ncm_dataset_log_info (mc->fit->lh->dset);
       ncm_cfg_msg_sepa ();
       g_message ("# NcmFitMC: Fiducial model set:\n");
@@ -600,34 +585,34 @@ ncm_fit_mc_start_run (NcmFitMC *mc)
       break;
   }
 
-  if (mc->rng == NULL)
+  if (mc->mcat->rng == NULL)
   {
     NcmRNG *rng = ncm_rng_new (NULL);
     ncm_rng_set_random_seed (rng, FALSE);
     ncm_fit_mc_set_rng (mc, rng);
     if (mc->mtype > NCM_FIT_RUN_MSGS_NONE)
-      g_message ("# NcmFitMC: No PRNG was defined, using algorithm: `%s' and seed: %lu.\n",
+      g_message ("# NcmFitMC: No RNG was defined, using algorithm: `%s' and seed: %lu.\n",
                  ncm_rng_get_algo (rng), ncm_rng_get_seed (rng));
     ncm_rng_free (rng);
   }
-  
+
   mc->started = TRUE;
 
-  ncm_fit_catalog_sync (mc->fcat, TRUE);
-  if (mc->fcat->cur_id > mc->cur_sample_id)
+  ncm_mset_catalog_sync (mc->mcat, TRUE);
+  if (mc->mcat->cur_id > mc->cur_sample_id)
   {
-    ncm_fit_mc_intern_skip (mc, mc->fcat->cur_id - mc->cur_sample_id);
-    g_assert_cmpint (mc->cur_sample_id, ==, mc->fcat->cur_id);
+    ncm_fit_mc_intern_skip (mc, mc->mcat->cur_id - mc->cur_sample_id);
+    g_assert_cmpint (mc->cur_sample_id, ==, mc->mcat->cur_id);
   }
-  else if (mc->fcat->cur_id < mc->cur_sample_id)
+  else if (mc->mcat->cur_id < mc->cur_sample_id)
     g_error ("ncm_fit_mc_set_data_file: Unknown error cur_id < cur_sample_id [%d < %d].", 
-             mc->fcat->cur_id, mc->cur_sample_id);
+             mc->mcat->cur_id, mc->cur_sample_id);
   
 }
 
 /**
  * ncm_fit_mc_end_run:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * 
  * FIXME
  * 
@@ -644,14 +629,14 @@ ncm_fit_mc_end_run (NcmFitMC *mc)
     mc->mp = NULL;
   }
 
-  ncm_fit_catalog_sync (mc->fcat, TRUE);
+  ncm_mset_catalog_sync (mc->mcat, TRUE);
   
   mc->started = FALSE;
 }
 
 /**
  * ncm_fit_mc_reset:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * 
  * FIXME
  * 
@@ -663,15 +648,12 @@ ncm_fit_mc_reset (NcmFitMC *mc)
   mc->cur_sample_id   = -1;
   mc->write_index     = 0;
   mc->started         = FALSE;  
-  ncm_fit_catalog_reset (mc->fcat);
-  ncm_rng_clear (&mc->rng);
+  ncm_mset_catalog_reset (mc->mcat);
 }
 
 static void 
 ncm_fit_mc_intern_skip (NcmFitMC *mc, guint n)
 {
-  guint i;
-
   if (n == 0)
     return;
 
@@ -682,33 +664,20 @@ ncm_fit_mc_intern_skip (NcmFitMC *mc, guint n)
     case NCM_FIT_RUN_MSGS_SIMPLE:
     {
       ncm_cfg_msg_sepa ();
-      g_message ("# NcmFitMC: Skipping %u realizations, will start at %u-th realization.\n#", n, mc->cur_sample_id + n + 1 + 1);
+      g_message ("# NcmFitMC: Skipping %u realizations, will start at %u-th realization.\n", n, mc->cur_sample_id + n + 1 + 1);
     }
     case NCM_FIT_RUN_MSGS_NONE:
       break;
   }
-  
-  for (i = 0; i < n; i++)
-  {
-    guint part = (n / 100) == 0 ? 1 : (n / 100);
-    _ncm_fit_mc_resample (mc, mc->fit);
-    if (mc->mtype > NCM_FIT_RUN_MSGS_NONE)
-    {
-      if (i % part == 0)
-        g_message (".");
-    }
-  }
 
+  mc->cur_sample_id += n;
   mc->write_index = mc->cur_sample_id + 1;
-
-  if (n > 0 && mc->mtype > NCM_FIT_RUN_MSGS_NONE)
-    g_message ("\n");
 }
 
 
 /**
  * ncm_fit_mc_set_first_sample_id:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * @first_sample_id: FIXME
  * 
  * FIXME
@@ -717,7 +686,7 @@ ncm_fit_mc_intern_skip (NcmFitMC *mc, guint n)
 void 
 ncm_fit_mc_set_first_sample_id (NcmFitMC *mc, gint first_sample_id)
 {
-  if (mc->fcat->first_id == first_sample_id)
+  if (mc->mcat->first_id == first_sample_id)
     return;
 
   if (!mc->started)
@@ -725,9 +694,9 @@ ncm_fit_mc_set_first_sample_id (NcmFitMC *mc, gint first_sample_id)
 
   if (first_sample_id <= mc->cur_sample_id)
     g_error ("ncm_fit_mc_set_first_sample_id: cannot move first sample id backwards to: %d, catalog first id: %d, current sample id: %d.",
-             first_sample_id, mc->fcat->first_id, mc->cur_sample_id);
+             first_sample_id, mc->mcat->first_id, mc->cur_sample_id);
 
-  ncm_fit_catalog_set_first_id (mc->fcat, first_sample_id);
+  ncm_mset_catalog_set_first_id (mc->mcat, first_sample_id);
   ncm_fit_mc_intern_skip (mc, first_sample_id - mc->cur_sample_id - 1);
 }
 
@@ -739,7 +708,7 @@ static void _ncm_fit_mc_run_mt (NcmFitMC *mc);
  * @mc: a #NcmFitMC
  * @n: total number of realizations to run
  * 
- * Runs the montecarlo until it reaches the @n-th realization. Note that
+ * Runs the Monte Carlo until it reaches the @n-th realization. Note that
  * if the first_id is non-zero it will run @n - first_id realizations.
  *
  */
@@ -754,7 +723,7 @@ ncm_fit_mc_run (NcmFitMC *mc, guint n)
     if (mc->mtype > NCM_FIT_RUN_MSGS_NONE)
     {
       ncm_cfg_msg_sepa ();
-      g_message ("# NcmFitMC: Nothing to do, current Montecarlo run is %d\n", mc->cur_sample_id + 1);
+      g_message ("# NcmFitMC: Nothing to do, current Monte Carlo run is %d\n", mc->cur_sample_id + 1);
     }
     return;
   }
@@ -769,7 +738,7 @@ ncm_fit_mc_run (NcmFitMC *mc, guint n)
     {
       const GEnumValue *eval = ncm_cfg_enum_get_value (NCM_TYPE_FIT_MC_RESAMPLE_TYPE, mc->rtype);
       ncm_cfg_msg_sepa ();
-      g_message ("# NcmFitMC: Calculating [%06d] Montecarlo fits [%s]\n", mc->n, eval->value_nick);
+      g_message ("# NcmFitMC: Calculating [%06d] Monte Carlo fits [%s]\n", mc->n, eval->value_nick);
     }
     case NCM_FIT_RUN_MSGS_NONE:
       break;
@@ -819,14 +788,14 @@ _ncm_fit_mc_dup_fit (gpointer userdata)
   g_mutex_lock (mc->dup_fit);
   {
     NcmFit *fit = ncm_fit_dup (mc->fit, mc->ser);
-    ncm_serialize_reset (mc->ser);
+    ncm_serialize_clear_instances (mc->ser);
     g_mutex_unlock (mc->dup_fit);
     return fit;
   }
 }
 
 static void 
-_ncm_fit_mc_mt_eval (glong i, glong f, gpointer data)
+_ncm_fit_mc_mt_eval_keep_order (glong i, glong f, gpointer data)
 {
   _NCM_STATIC_MUTEX_DECL (update_lock);
   NcmFitMC *mc = NCM_FIT_MC (data);
@@ -860,6 +829,36 @@ _ncm_fit_mc_mt_eval (glong i, glong f, gpointer data)
   ncm_memory_pool_return (fit_ptr);
 }
 
+
+static void 
+_ncm_fit_mc_mt_eval (glong i, glong f, gpointer data)
+{
+  _NCM_STATIC_MUTEX_DECL (update_lock);
+  NcmFitMC *mc = NCM_FIT_MC (data);
+  NcmFit **fit_ptr = ncm_memory_pool_get (mc->mp);
+  NcmFit *fit = *fit_ptr;
+  guint j;
+
+  for (j = i; j < f; j++)
+  {
+    ncm_mset_param_set_vector (fit->mset, mc->bf);
+
+    g_mutex_lock (mc->resample_lock);
+    _ncm_fit_mc_resample (mc, fit);
+    g_mutex_unlock (mc->resample_lock);
+
+    ncm_fit_run (fit, NCM_FIT_RUN_MSGS_NONE);
+
+    _NCM_MUTEX_LOCK (&update_lock);    
+    _ncm_fit_mc_update (mc, fit);
+    mc->write_index++;
+    _NCM_MUTEX_UNLOCK (&update_lock);
+  }
+
+  ncm_memory_pool_return (fit_ptr);
+}
+
+
 static void
 _ncm_fit_mc_run_mt (NcmFitMC *mc)
 {
@@ -885,12 +884,15 @@ _ncm_fit_mc_run_mt (NcmFitMC *mc)
 
   g_assert_cmpuint (mc->nthreads, >, 1);
 
-  ncm_func_eval_threaded_loop_nw (&_ncm_fit_mc_mt_eval, 0, mc->n, mc, nthreads);
+  if (mc->keep_order)
+    ncm_func_eval_threaded_loop_full (&_ncm_fit_mc_mt_eval_keep_order, 0, mc->n, mc);
+  else
+    ncm_func_eval_threaded_loop_full (&_ncm_fit_mc_mt_eval, 0, mc->n, mc);  
 }
 
 /**
  * ncm_fit_mc_run_lre:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  * @prerun: FIXME
  * @lre: FIXME
  *
@@ -908,20 +910,20 @@ ncm_fit_mc_run_lre (NcmFitMC *mc, guint prerun, gdouble lre)
   
   prerun = (prerun == 0) ? 100 : prerun;
 
-  if (ncm_fit_catalog_len (mc->fcat) < prerun)
+  if (ncm_mset_catalog_len (mc->mcat) < prerun)
   {
-    guint prerun_left = prerun - ncm_fit_catalog_len (mc->fcat);
+    guint prerun_left = prerun - ncm_mset_catalog_len (mc->mcat);
     if (mc->mtype >= NCM_FIT_RUN_MSGS_SIMPLE)
       g_message ("# NcmFitMC: Running first %u pre-runs...\n", prerun_left);
     ncm_fit_mc_run (mc, prerun);
   }
 
-  lerror = ncm_fit_catalog_largest_error (mc->fcat);
+  lerror = ncm_mset_catalog_largest_error (mc->mcat);
 
   while (lerror > lre)
   {
     const gdouble lerror2 = lerror * lerror;
-    gdouble n = ncm_fit_catalog_len (mc->fcat);
+    gdouble n = ncm_mset_catalog_len (mc->mcat);
     gdouble m = n * lerror2 / lre2;
     guint runs = ((m - n) > 1000.0) ? ceil ((m - n) * 0.25) : ceil (m - n);
 
@@ -931,7 +933,7 @@ ncm_fit_mc_run_lre (NcmFitMC *mc, guint prerun, gdouble lre)
       g_message ("# NcmFitMC: Running more %u runs...\n", runs);
     }
     ncm_fit_mc_run (mc, mc->cur_sample_id + runs + 1);
-    lerror = ncm_fit_catalog_largest_error (mc->fcat);
+    lerror = ncm_mset_catalog_largest_error (mc->mcat);
   }
 
   if (mc->mtype >= NCM_FIT_RUN_MSGS_SIMPLE)
@@ -940,12 +942,29 @@ ncm_fit_mc_run_lre (NcmFitMC *mc, guint prerun, gdouble lre)
 
 /**
  * ncm_fit_mc_mean_covar:
- * @mc: FIXME
+ * @mc: a #NcmFitMC
  *
  * FIXME
  */
 void
 ncm_fit_mc_mean_covar (NcmFitMC *mc)
 {
-  ncm_fit_catalog_set_fit_mean_covar (mc->fcat);
+  ncm_mset_catalog_get_mean (mc->mcat, &mc->fit->fstate->fparams);
+  ncm_mset_catalog_get_covar (mc->mcat, &mc->fit->fstate->covar);
+  ncm_mset_fparams_set_vector (mc->mcat->mset, mc->fit->fstate->fparams);
+  mc->fit->fstate->has_covar = TRUE;
+}
+
+/**
+ * ncm_fit_mc_get_catalog:
+ * @mc: a #NcmFitMC
+ *
+ * Gets the generated catalog of @mc.
+ * 
+ * Returns: (transfer full): the generated catalog.
+ */
+NcmMSetCatalog *
+ncm_fit_mc_get_catalog (NcmFitMC *mc)
+{
+  return ncm_mset_catalog_ref (mc->mcat);
 }
