@@ -25,10 +25,11 @@
 
 /**
  * SECTION:ncm_fit_state
- * @title: Fitting State
- * @short_description: Object representing the current state of a NcmFit object
+ * @title: NcmFitState
+ * @short_description: State of a NcmFit object.
  *
  * FIXME
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -58,17 +59,19 @@ G_DEFINE_TYPE (NcmFitState, ncm_fit_state, G_TYPE_OBJECT);
 static void
 ncm_fit_state_init (NcmFitState *fstate)
 {
-  fstate->data_len   = 0;
-  fstate->fparam_len = 0;
-  fstate->covar      = NULL;
-  fstate->hessian    = NULL;
-  fstate->fparams    = NULL;
-  fstate->dm2lnL     = NULL;
-  fstate->ls_J       = NULL;
-  fstate->m2lnL      = 0.0;
-  fstate->niter      = 0;
-  fstate->func_eval  = 0;
-  fstate->grad_eval  = 0;
+  fstate->data_len     = 0;
+  fstate->fparam_len   = 0;
+  fstate->covar        = NULL;
+  fstate->hessian      = NULL;
+  fstate->fparams      = NULL;
+  fstate->dm2lnL       = NULL;
+  fstate->ls_J         = NULL;
+  fstate->m2lnL_curval = 0.0;
+  fstate->niter        = 0;
+  fstate->func_eval    = 0;
+  fstate->grad_eval    = 0;
+  fstate->m2lnL_prec   = 0.0;
+  fstate->params_prec  = 0.0;
 
   fstate->alloc_data_len   = 0;
   fstate->alloc_fparam_len = 0;
@@ -86,7 +89,7 @@ ncm_fit_state_constructed (GObject *object)
   G_OBJECT_CLASS (ncm_fit_state_parent_class)->constructed (object);
   {
     NcmFitState *fstate = NCM_FIT_STATE (object);
-    ncm_fit_state_reset (fstate);
+    ncm_fit_state_realloc (fstate);
   }
 }
 
@@ -100,15 +103,18 @@ ncm_fit_state_set_property (GObject *object, guint prop_id, const GValue *value,
   {
     case PROP_DATA_LEN:
       fstate->data_len = g_value_get_uint (value);
+      ncm_fit_state_realloc (fstate);
       break;
     case PROP_FPARAM_LEN:
       fstate->fparam_len = g_value_get_uint (value);
+      ncm_fit_state_realloc (fstate);
       break;
     case PROP_DOF:
       fstate->dof = g_value_get_int (value);
       break;
     case PROP_IS_LEAST_SQUARES:
       fstate->is_least_squares = g_value_get_boolean (value);
+      ncm_fit_state_realloc (fstate);
       break;
     case PROP_NITERS:
       fstate->niter = g_value_get_uint (value);
@@ -329,7 +335,6 @@ ncm_fit_state_clear (NcmFitState **fstate)
  * 
  * FIXME
  * 
- * Returns: FIXME
  */
 void 
 ncm_fit_state_set_all (NcmFitState *fstate, guint data_len, guint fparam_len, gint dof, gboolean is_least_squares)
@@ -338,6 +343,8 @@ ncm_fit_state_set_all (NcmFitState *fstate, guint data_len, guint fparam_len, gi
   fstate->fparam_len       = fparam_len;
   fstate->dof              = dof;
   fstate->is_least_squares = is_least_squares;
+  
+  ncm_fit_state_realloc (fstate);
 }
 
 /**
@@ -346,22 +353,35 @@ ncm_fit_state_set_all (NcmFitState *fstate, guint data_len, guint fparam_len, gi
  * 
  * FIXME
  * 
- * Returns: FIXME
  */
 void 
 ncm_fit_state_reset (NcmFitState *fstate)
 {
-  gboolean fparam_diff_len = FALSE;
-
   fstate->niter        = 0;
   fstate->func_eval    = 0;
   fstate->grad_eval    = 0;
   fstate->elapsed_time = 0.0;
   fstate->m2lnL_prec   = 0.0;
   fstate->params_prec  = 0.0;
-  fstate->m2lnL        = 0.0;
+  fstate->m2lnL_curval = 0.0;
 
   fstate->is_best_fit = FALSE;
+}
+
+/**
+ * ncm_fit_state_realloc:
+ * @fstate: FIXME
+ * 
+ * FIXME
+ * 
+ */
+void 
+ncm_fit_state_realloc (NcmFitState *fstate)
+{
+  gboolean fparam_diff_len = FALSE;
+
+  if (fstate->data_len == 0)
+    return;
   
   if (fstate->alloc_fparam_len != fstate->fparam_len)
   {
@@ -386,18 +406,98 @@ ncm_fit_state_reset (NcmFitState *fstate)
   {
     gboolean data_diff_len = fstate->alloc_data_len != fstate->data_len;
     
-    if (fparam_diff_len || data_diff_len)
+    if ((fparam_diff_len || data_diff_len || fstate->ls_J == NULL) && fstate->fparam_len > 0)
     {
       ncm_matrix_clear (&fstate->ls_J);
       fstate->ls_J = ncm_matrix_new (fstate->data_len, fstate->fparam_len);
     }
     
-    if (data_diff_len)
+    if (data_diff_len || fstate->ls_f == NULL)
     {
       ncm_vector_clear (&fstate->ls_f);
       fstate->ls_f = ncm_vector_new (fstate->data_len);
     }
   }
-
+  else
+  {
+    ncm_matrix_clear (&fstate->ls_J);
+    ncm_vector_clear (&fstate->ls_f);
+  }
+  
   fstate->alloc_data_len = fstate->data_len;
 }
+
+/**
+ * ncm_fit_state_set_ls:
+ * @fstate: FIXME
+ * @f: FIXME
+ * @J: FIXME
+ * 
+ * FIXME
+ * 
+ */
+/**
+ * ncm_fit_state_set_niter:
+ * @fstate: FIXME
+ * @niter: FIXME
+ * 
+ * FIXME
+ * 
+ */
+/**
+ * ncm_fit_state_get_niter:
+ * @fstate: FIXME
+ * 
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+/**
+ * ncm_fit_state_set_m2lnL_prec:
+ * @fstate: FIXME
+ * @prec: FIXME
+ * 
+ * FIXME
+ * 
+ */
+/**
+ * ncm_fit_state_get_m2lnL_prec:
+ * @fstate: FIXME
+ * 
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+/**
+ * ncm_fit_state_set_m2lnL_curval:
+ * @fstate: FIXME
+ * @m2lnL: FIXME
+ * 
+ * FIXME
+ * 
+ */
+/**
+ * ncm_fit_state_get_m2lnL_curval:
+ * @fstate: FIXME
+ * 
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+/**
+ * ncm_fit_state_set_params_prec:
+ * @fstate: FIXME
+ * @prec: FIXME
+ * 
+ * FIXME
+ * 
+ */
+/**
+ * ncm_fit_state_get_params_prec:
+ * @fstate: FIXME
+ * 
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+
