@@ -55,12 +55,6 @@ typedef enum _NcmLHRatio2dRoot
   NCM_LH_RATIO2D_ROOT_NUMDIFF,
 } NcmLHRatio2dRoot;
 
-struct _NcmLHRatio2dClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-};
-
 struct _NcmLHRatio2d
 {
   /*< private >*/
@@ -70,10 +64,12 @@ struct _NcmLHRatio2d
   NcmFitRunMsgs mtype;
   NcmLHRatio2dRoot rtype;
   NcmMSetPIndex pi[2];
+  NcmRNG *rng;
   gdouble chisquare;
   gdouble lb[2];
   gdouble ub[2];
   gdouble bf[2];
+  gdouble border_prec;
   NcmMatrix *e_vec;
   NcmVector *e_val;
   gdouble r, theta;
@@ -84,6 +80,12 @@ struct _NcmLHRatio2d
   guint grad_eval;
 };
 
+struct _NcmLHRatio2dClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+};
+
 typedef struct _NcmLHRatio2dPoint NcmLHRatio2dPoint;
 
 /**
@@ -124,7 +126,7 @@ struct _NcmLHRatio2dRegion
 GType ncm_lh_ratio2d_get_type (void) G_GNUC_CONST;
 GType ncm_lh_ratio2d_region_get_type (void) G_GNUC_CONST;
 
-NcmLHRatio2d *ncm_lh_ratio2d_new (NcmFit *fit, NcmMSetPIndex *pi1, NcmMSetPIndex *pi2);
+NcmLHRatio2d *ncm_lh_ratio2d_new (NcmFit *fit, NcmMSetPIndex *pi1, NcmMSetPIndex *pi2, gdouble border_prec);
 void ncm_lh_ratio2d_free (NcmLHRatio2d *lhr2d);
 void ncm_lh_ratio2d_clear (NcmLHRatio2d **lhr2d);
 
