@@ -65,6 +65,7 @@ struct _NcmModel
   GHashTable *sparams_name_id;
   guint total_len;
   guint64 pkey;
+  guint64 skey;
 };
 
 struct _NcmModelClass
@@ -89,13 +90,14 @@ struct _NcmModelClass
 
 typedef gdouble (*NcmModelFunc0) (NcmModel *model);
 typedef gdouble (*NcmModelFunc1) (NcmModel *model, const gdouble x);
+typedef gdouble (*NcmModelFunc2) (NcmModel *model, const gdouble x, const gdouble y);
 
 GType ncm_model_get_type (void) G_GNUC_CONST;
 
 void ncm_model_class_add_params (NcmModelClass *model_class, guint sparam_len, guint vparam_len, guint nonparam_prop_len);
 void ncm_model_class_set_name_nick (NcmModelClass *model_class, const gchar *name, const gchar *nick);
-void ncm_model_class_set_sparam (NcmModelClass *model_class, guint sparam_id, gchar *symbol, gchar *name, gdouble lower_bound, gdouble upper_bound, gdouble scale, gdouble abstol, gdouble default_value, NcmParamType ppt);
-void ncm_model_class_set_vparam (NcmModelClass *model_class, guint vparam_id, guint default_length, gchar *symbol, gchar *name, gdouble lower_bound, gdouble upper_bound, gdouble scale, gdouble abstol, gdouble default_value, NcmParamType ppt);
+void ncm_model_class_set_sparam (NcmModelClass *model_class, guint sparam_id, const gchar *symbol, const gchar *name, gdouble lower_bound, gdouble upper_bound, gdouble scale, gdouble abstol, gdouble default_value, NcmParamType ppt);
+void ncm_model_class_set_vparam (NcmModelClass *model_class, guint vparam_id, guint default_length, const gchar *symbol, const gchar *name, gdouble lower_bound, gdouble upper_bound, gdouble scale, gdouble abstol, gdouble default_value, NcmParamType ppt);
 void ncm_model_class_check_params_info (NcmModelClass *model_class);
 
 NcmModel *ncm_model_dup (NcmModel *model, NcmSerialize *ser);
@@ -108,7 +110,11 @@ G_INLINE_FUNC NcmModel *ncm_model_ref (NcmModel *model);
 G_INLINE_FUNC NcmModelID ncm_model_id (NcmModel *model);
 G_INLINE_FUNC NcmModelID ncm_model_id_by_type (GType model_type);
 G_INLINE_FUNC guint64 ncm_model_impl (NcmModel *model);
+G_INLINE_FUNC gboolean ncm_model_check_impl (NcmModel *model, guint64 impl);
 G_INLINE_FUNC guint ncm_model_len (NcmModel *model);
+G_INLINE_FUNC gboolean ncm_model_state_is_update (NcmModel *model);
+G_INLINE_FUNC void ncm_model_state_set_update (NcmModel *model);
+
 G_INLINE_FUNC guint ncm_model_sparam_len (NcmModel *model);
 G_INLINE_FUNC guint ncm_model_vparam_array_len (NcmModel *model);
 G_INLINE_FUNC guint ncm_model_vparam_index (NcmModel *model, guint n, guint i);
@@ -143,14 +149,20 @@ void ncm_model_params_print_all (NcmModel *model, FILE *out);
 void ncm_model_params_log_all (NcmModel *model);
 NcmVector *ncm_model_params_get_all (NcmModel *model);
 gboolean ncm_model_params_valid (NcmModel *model);
+gboolean ncm_model_params_valid_bounds (NcmModel *model);
 
-gboolean ncm_model_orig_param_index_from_name (NcmModel *model, gchar *param_name, guint *i);
-gboolean ncm_model_param_index_from_name (NcmModel *model, gchar *param_name, guint *i);
+gboolean ncm_model_orig_param_index_from_name (NcmModel *model, const gchar *param_name, guint *i);
+gboolean ncm_model_param_index_from_name (NcmModel *model, const gchar *param_name, guint *i);
 const gchar *ncm_model_orig_param_name (NcmModel *model, guint n);
 const gchar *ncm_model_param_name (NcmModel *model, guint n);
 const gchar *ncm_model_orig_param_symbol (NcmModel *model, guint n);
 const gchar *ncm_model_param_symbol (NcmModel *model, guint n);
 
+void ncm_model_param_set_by_name (NcmModel *model, const gchar *param_name, gdouble val);
+void ncm_model_orig_param_set_by_name (NcmModel *model, const gchar *param_name, gdouble val);
+gdouble ncm_model_param_get_by_name (NcmModel *model, const gchar *param_name);
+gdouble ncm_model_orig_param_get_by_name (NcmModel *model, const gchar *param_name);
+
 gdouble ncm_model_orig_param_get_scale (NcmModel *model, guint n);
 gdouble ncm_model_orig_param_get_lower_bound (NcmModel *model, guint n);
 gdouble ncm_model_orig_param_get_upper_bound (NcmModel *model, guint n);
@@ -201,6 +213,15 @@ G_INLINE_FUNC gdouble ns_name##_##name (NsName *m, const gdouble x) \
   return NS_NAME##_GET_CLASS (m)->name (NCM_MODEL (m), x); \
 }
 
+/*
+ * Model functions 2d call
+ */
+#define NCM_MODEL_FUNC2_IMPL(NS_NAME,NsName,ns_name,name) \
+G_INLINE_FUNC gdouble ns_name##_##name (NsName *m, const gdouble x, const gdouble y) \
+{ \
+  return NS_NAME##_GET_CLASS (m)->name (NCM_MODEL (m), x, y); \
+}
+
 G_END_DECLS
 
 #endif /* _NCM_MODEL_H_ */
@@ -246,12 +267,33 @@ ncm_model_impl (NcmModel *model)
   return NCM_MODEL_GET_CLASS (model)->impl;
 }
 
+G_INLINE_FUNC gboolean 
+ncm_model_check_impl (NcmModel *model, guint64 impl)
+{
+  if (impl == 0)
+    return TRUE;
+  else
+    return ((NCM_MODEL_GET_CLASS (model)->impl & impl) == impl);
+}
+
 G_INLINE_FUNC guint
 ncm_model_len (NcmModel *model)
 {
   return ncm_vector_len (model->params);
 }
 
+G_INLINE_FUNC gboolean
+ncm_model_state_is_update (NcmModel *model)
+{
+  return model->pkey == model->skey;
+}
+
+G_INLINE_FUNC void
+ncm_model_state_set_update (NcmModel *model)
+{
+  model->skey = model->pkey;
+}
+
 G_INLINE_FUNC guint
 ncm_model_sparam_len (NcmModel *model)
 {
