@@ -24,10 +24,11 @@
 
 /**
  * SECTION:ncm_model_ctrl
- * @title: Model Update Control Object
- * @short_description: Control object for testing updates on model status
+ * @title: NcmModelCtrl
+ * @short_description: Control object for testing updates on model status.
  *
  * FIXME
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +37,7 @@
 #include "build_cfg.h"
 
 #include "math/ncm_model_ctrl.h"
+#include "math/ncm_cfg.h"
 
 enum
 {
@@ -82,9 +84,9 @@ NcmModelCtrl *
 ncm_model_ctrl_new (NcmModel *model)
 {
   if (model)
-	return g_object_new (NCM_TYPE_MODEL_CTRL, "model", model, NULL);
+    return g_object_new (NCM_TYPE_MODEL_CTRL, "model", model, NULL);
   else
-	return g_object_new (NCM_TYPE_MODEL_CTRL, NULL);
+    return g_object_new (NCM_TYPE_MODEL_CTRL, NULL);
 }
 
 /**
@@ -111,7 +113,15 @@ ncm_model_ctrl_copy (NcmModelCtrl *ctrl)
  *
  * Returns: FIXME
  */
-
+/**
+ * ncm_model_ctrl_model_update:
+ * @ctrl: FIXME
+ * @model: FIXME
+ *
+ * FIXME
+ *
+ * Returns: FIXME
+ */
 /**
  * ncm_model_ctrl_get_model:
  * @ctrl: FIXME
