@@ -60,14 +60,6 @@ struct _NcmMSetModelDesc
   gchar *long_desc;
 };
 
-struct _NcmMSetClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-  GHashTable *ns_table;
-  NcmMSetModelDesc model_desc[NCM_MODEL_MAX_ID];
-};
-
 struct _NcmMSet
 {
   /*< private >*/
@@ -75,11 +67,20 @@ struct _NcmMSet
   NcmModel *model[NCM_MODEL_MAX_ID];
   GArray *pi_array;
   GArray *fpi_array[NCM_MODEL_MAX_ID];
+  GPtrArray *fullname_parray;
   gboolean valid_map;
   guint total_len;
   guint fparam_len;
 };
 
+struct _NcmMSetClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+  GHashTable *ns_table;
+  NcmMSetModelDesc model_desc[NCM_MODEL_MAX_ID];
+};
+
 struct _NcmMSetPIndex
 {
   /*< private >*/
@@ -90,7 +91,7 @@ struct _NcmMSetPIndex
 GType ncm_mset_get_type (void) G_GNUC_CONST;
 GType ncm_mset_pindex_get_type (void) G_GNUC_CONST;
 
-void ncm_mset_model_register_id (NcmModelClass *model_class, gchar *ns, gchar *desc, gchar *long_desc);
+void ncm_mset_model_register_id (NcmModelClass *model_class, const gchar *ns, const gchar *desc, const gchar *long_desc);
 
 /**
  * NCM_MSET_MODEL_ID_FUNC: (skip)
@@ -140,10 +141,8 @@ NcmMSet *ncm_mset_new (NcmModel *model0, ...);
 NcmMSet *ncm_mset_newv (NcmModel *model0, va_list ap);
 NcmMSet *ncm_mset_new_array (NcmModel **model);
 NcmMSet *ncm_mset_ref (NcmMSet *mset);
-NcmMSet *ncm_mset_dup (NcmMSet *mset);
-NcmMSet *ncm_mset_copy (NcmMSet *mset);
+NcmMSet *ncm_mset_dup (NcmMSet *mset, NcmSerialize *ser);
 
-void ncm_mset_copyto (NcmMSet *mset_src, NcmMSet *mset_dest);
 void ncm_mset_free (NcmMSet *mset);
 void ncm_mset_clear (NcmMSet **mset);
 
@@ -153,6 +152,8 @@ void ncm_mset_remove (NcmMSet *mset, NcmModelID mid);
 void ncm_mset_set (NcmMSet *mset, NcmModel *model);
 gboolean ncm_mset_exists (NcmMSet *mset, NcmModel *model);
 
+gint ncm_mset_get_id_by_ns (const gchar *ns);
+
 void ncm_mset_prepare_fparam_map (NcmMSet *mset);
 
 guint ncm_mset_total_len (NcmMSet *mset);
@@ -162,10 +163,13 @@ guint ncm_mset_max_fparam_name (NcmMSet *mset);
 guint ncm_mset_max_model_nick (NcmMSet *mset);
 
 void ncm_mset_pretty_log (NcmMSet *mset);
-void ncm_mset_params_pretty_print (NcmMSet *mset, FILE *out, gchar *header);
+void ncm_mset_params_pretty_print (NcmMSet *mset, FILE *out, const gchar *header);
 void ncm_mset_params_log_vals (NcmMSet *mset);
 void ncm_mset_params_print_vals (NcmMSet *mset, FILE *out);
+void ncm_mset_fparams_log_covar (NcmMSet *mset, NcmMatrix *covar);
+
 gboolean ncm_mset_params_valid (NcmMSet *mset);
+gboolean ncm_mset_params_valid_bounds (NcmMSet *mset);
 gboolean ncm_mset_cmp (NcmMSet *mset0, NcmMSet *mset1, gboolean cmp_model);
 
 void ncm_mset_param_set (NcmMSet *mset, NcmModelID mid, guint pid, const gdouble x);
@@ -174,6 +178,7 @@ gdouble ncm_mset_orig_param_get (NcmMSet *mset, NcmModelID mid, guint pid);
 
 guint ncm_mset_param_len (NcmMSet *mset);
 const gchar *ncm_mset_param_name (NcmMSet *mset, NcmModelID mid, guint pid);
+const gchar *ncm_mset_param_symbol (NcmMSet *mset, NcmModelID mid, guint pid);
 void ncm_mset_param_set_ftype (NcmMSet *mset, NcmModelID mid, guint pid, NcmParamType ftype);
 void ncm_mset_param_set_all_ftype (NcmMSet *mset, NcmParamType ftype);
 void ncm_mset_param_set_vector (NcmMSet *mset, NcmVector *params);
@@ -183,21 +188,28 @@ gdouble ncm_mset_param_get_lower_bound (NcmMSet *mset, NcmModelID mid, guint pid
 gdouble ncm_mset_param_get_upper_bound (NcmMSet *mset, NcmModelID mid, guint pid);
 gdouble ncm_mset_param_get_abstol (NcmMSet *mset, NcmModelID mid, guint pid);
 NcmParamType ncm_mset_param_get_ftype (NcmMSet *mset, NcmModelID mid, guint pid);
+void ncm_mset_param_set_scale (NcmMSet *mset, NcmModelID mid, guint pid, gdouble scale);
 
 void ncm_mset_param_set_pi (NcmMSet *mset, NcmMSetPIndex *pi, const gdouble *x, guint n);
 void ncm_mset_param_get_pi (NcmMSet *mset, NcmMSetPIndex *pi, gdouble *x, guint n);
 
 void ncm_mset_fparams_get_vector (NcmMSet *mset, NcmVector *x);
+void ncm_mset_fparams_get_vector_offset (NcmMSet *mset, NcmVector *x, guint offset);
 void ncm_mset_fparams_set_vector (NcmMSet *mset, const NcmVector *x);
+void ncm_mset_fparams_set_vector_offset (NcmMSet *mset, const NcmVector *x, guint offset);
 void ncm_mset_fparams_set_array (NcmMSet *mset, const gdouble *x);
 void ncm_mset_fparams_set_gsl_vector (NcmMSet *mset, const gsl_vector *x);
 
 guint ncm_mset_fparams_len (NcmMSet *mset);
 const gchar *ncm_mset_fparam_name (NcmMSet *mset, guint n);
+const gchar *ncm_mset_fparam_symbol (NcmMSet *mset, guint n);
+const gchar *ncm_mset_fparam_full_name (NcmMSet *mset, guint n);
 gdouble ncm_mset_fparam_get_scale (NcmMSet *mset, guint n);
 gdouble ncm_mset_fparam_get_lower_bound (NcmMSet *mset, guint n);
 gdouble ncm_mset_fparam_get_upper_bound (NcmMSet *mset, guint n);
 gdouble ncm_mset_fparam_get_abstol (NcmMSet *mset, guint n);
+void ncm_mset_fparam_set_scale (NcmMSet *mset, guint n, gdouble scale);
+gboolean ncm_mset_fparam_valid_bounds (NcmMSet *mset, NcmVector *theta);
 
 gdouble ncm_mset_fparam_get (NcmMSet *mset, guint n);
 void ncm_mset_fparam_set (NcmMSet *mset, guint n, const gdouble x);
@@ -205,8 +217,10 @@ void ncm_mset_fparam_set (NcmMSet *mset, guint n, const gdouble x);
 NcmMSetPIndex *ncm_mset_fparam_get_pi (NcmMSet *mset, guint n);
 gint ncm_mset_fparam_get_fpi (NcmMSet *mset, NcmModelID mid, guint pid);
 
-void ncm_mset_save (NcmMSet *mset, gchar *filename, gboolean save_comment);
-NcmMSet *ncm_mset_load (gchar *filename);
+NcmMSetPIndex *ncm_mset_fparam_get_pi_by_name (NcmMSet *mset, const gchar *name);
+
+void ncm_mset_save (NcmMSet *mset, NcmSerialize *ser, const gchar *filename, gboolean save_comment);
+NcmMSet *ncm_mset_load (const gchar *filename, NcmSerialize *ser);
 
 G_END_DECLS
 
