@@ -66,12 +66,6 @@ struct __NcmSpline2dBicubicOptimizeInt
   NcmSpline *s;
 };
 
-struct _NcmSpline2dBicubicClass
-{
-  /*< private >*/
-  NcmSpline2dClass parent_class;
-};
-
 struct _NcmSpline2dBicubic
 {
   /*< private >*/
@@ -84,10 +78,16 @@ struct _NcmSpline2dBicubic
   _NcmSpline2dBicubicOptimizeInt optimize_dy;
 };
 
+struct _NcmSpline2dBicubicClass
+{
+  /*< private >*/
+  NcmSpline2dClass parent_class;
+};
+
 GType ncm_spline2d_bicubic_get_type (void) G_GNUC_CONST;
 
 NcmSpline2d *ncm_spline2d_bicubic_new (NcmSpline *s);
-NcmSpline2d *ncm_spline2d_bicubic_notaknot_new ();
+NcmSpline2d *ncm_spline2d_bicubic_notaknot_new (void);
 
 #define NCM_SPLINE2D_BICUBIC_00 (0)
 #define NCM_SPLINE2D_BICUBIC_10 (1)
