@@ -32,6 +32,7 @@
 #include <gsl/gsl_rng.h>
 #include <gsl/gsl_min.h>
 #ifndef NUMCOSMO_GIR_SCAN
+#include <complex.h>
 #include <gmp.h>
 #include <mpfr.h>
 #endif
@@ -63,6 +64,32 @@ void ncm_mpz_inits (mpz_t z, ...);
 void ncm_mpz_clears (mpz_t z, ...);
 void _ncm_assertion_message_cmpdouble (const gchar *domain, const gchar *file, gint line, const gchar *func, const gchar *expr, gdouble arg1, const gchar *cmp, gdouble arg2);
 
+gboolean ncm_util_cvode_check_flag (gpointer flagvalue, const gchar *funcname, gint opt);
+gboolean ncm_util_cvode_print_stats (gpointer cvode);
+
+typedef struct _NcmComplex NcmComplex;
+
+struct _NcmComplex
+{
+  /*< private >*/
+#ifndef NUMCOSMO_GIR_SCAN
+  complex double z;
+#else
+  gdouble Rez, Imz;
+#endif /* NUMCOSMO_GIR_SCAN */
+};
+
+GType ncm_complex_get_type (void) G_GNUC_CONST;
+
+NcmComplex *ncm_complex_new (void);
+NcmComplex *ncm_complex_ref (NcmComplex *c);
+NcmComplex *ncm_complex_dup (NcmComplex *c);
+void ncm_complex_free (NcmComplex *c);
+void ncm_complex_clear (NcmComplex **c);
+
+gdouble ncm_complex_Re (NcmComplex *c);
+gdouble ncm_complex_Im (NcmComplex *c);
+
 /* Macros */
 
 #define NCM_GARRAY_MEMCPY(dest,src) \
@@ -168,6 +195,15 @@ G_STMT_START { \
 
 #define NCM_UNUSED(x) (void)(x)
 
+#define NCM_TEST_FREE(cmd,obj) \
+G_STMT_START { \
+  g_object_add_weak_pointer (G_OBJECT (obj), (gpointer *)&obj); \
+  cmd (obj); \
+  g_assert (obj == NULL); \
+} G_STMT_END
+
+/* Minumum version here is 2.38 but it segfault during tests so we start at 2.40. */
+#if ((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 40))
 #define NCM_TEST_FAIL(cmd) \
 G_STMT_START { \
   if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR)) \
@@ -188,6 +224,130 @@ G_STMT_START { \
   g_test_trap_assert_passed (); \
 } G_STMT_END
 
+#else
+
+#define NCM_TEST_FAIL(cmd) \
+G_STMT_START { \
+  if (g_test_subprocess ()) \
+  { \
+    cmd; \
+    exit (0); \
+  } \
+  else \
+  { \
+    g_test_trap_subprocess (NULL, 0, 0); \
+    g_test_trap_assert_failed (); \
+  } \
+} G_STMT_END
+
+#define NCM_TEST_PASS(cmd) \
+G_STMT_START { \
+  if (g_test_subprocess ()) \
+  { \
+    cmd; \
+    exit (0); \
+  } \
+  else \
+  { \
+    g_test_trap_subprocess (NULL, 0, 0); \
+    g_test_trap_assert_passed (); \
+  } \
+} G_STMT_END
+#endif /* !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 38)) */
+
+#define NCM_CVODE_CHECK(chk,name,val,ret) \
+G_STMT_START { \
+  if (!ncm_util_cvode_check_flag (chk, name, val)) \
+    return ret; \
+} G_STMT_END
+
 G_END_DECLS
 
 #endif /* _NCM_UTIL_H_ */
+
+#ifndef KOLMOGOROVSMIRNOVDIST_H
+#define KOLMOGOROVSMIRNOVDIST_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/********************************************************************
+ *
+ * File:          KolmogorovSmirnovDist.h
+ * Environment:   ISO C99 or ANSI C89
+ * Author:        Richard Simard
+ * Organization:  DIRO, Université de Montréal
+ * Date:          1 February 2012
+ * Version        1.1
+ *
+ * Copyright March 2010 by Université de Montréal,
+                           Richard Simard and Pierre L'Ecuyer
+ =====================================================================
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, version 3 of the License.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+ =====================================================================*/
+/*
+ *
+ * The Kolmogorov-Smirnov test statistic D_n is defined by
+ *
+ *        D_n = sup_x |F(x) - S_n(x)|
+ *
+ * where n is the sample size, F(x) is a completely specified theoretical
+ * distribution, and S_n(x) is an empirical distribution function.
+ *
+ *
+ * The function
+ *
+ *        double ncm_util_KScdf (int n, double x);
+ *
+ * computes the cumulative probability P[D_n <= x] of the 2-sided 1-sample
+ * Kolmogorov-Smirnov distribution with sample size n at x.
+ * It returns at least 13 decimal digits of precision for n <= 500,
+ * at least 7 decimal digits of precision for 500 < n <= 100000,
+ * and a few correct decimal digits for n > 100000.
+ *
+ */
+double ncm_util_KScdf (int n, double x);
+
+/*
+ * The function
+ *
+ *        double ncm_util_KSfbar (int n, double x);
+ *
+ * computes the complementary cumulative probability P[D_n >= x] of the
+ * 2-sided 1-sample Kolmogorov-Smirnov distribution with sample size n at x.
+ * It returns at least 10 decimal digits of precision for n <= 500,
+ * at least 6 decimal digits of precision for 500 < n <= 200000,
+ * and a few correct decimal digits for n > 200000.
+ *
+ */
+double ncm_util_KSfbar (int n, double x);
+
+/*
+ * NOTE:
+ * The ISO C99 function log1p of the standard math library does not exist in
+ * ANSI C89. Here, it is programmed explicitly in KolmogorovSmirnovDist.c.
+
+ * For ANSI C89 compilers, change the preprocessor condition to make it
+ * available.
+ */
+
+void ncm_util_swilk (double *x, int n, double *w, double *pw, int *ifault);
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif
