@@ -25,8 +25,8 @@
 
 /**
  * SECTION:ncm_vparam
- * @title: Model Vector Parameter Description
- * @short_description: Describes the properties of a vector parameter
+ * @title: NcmVParam
+ * @short_description: Properties of a vector-like parameter.
  *
  * This object comprises the necessary properties to define a vector parameter.
  * It is used by #NcmModel to store the description of the vector model parameters.
@@ -40,6 +40,7 @@
 
 #include "math/ncm_vparam.h"
 #include "math/ncm_cfg.h"
+#include "math/ncm_util.h"
 
 enum
 {
@@ -93,7 +94,7 @@ ncm_vparam_new (guint len, NcmSParam *default_param)
  * Returns: A new #NcmVParam.
  */
 NcmVParam *
-ncm_vparam_full_new (guint len, gchar *name, gchar *symbol, gdouble lower_bound, gdouble upper_bound, gdouble scale, gdouble abstol, gdouble default_val, NcmParamType ftype)
+ncm_vparam_full_new (guint len, const gchar *name, const gchar *symbol, gdouble lower_bound, gdouble upper_bound, gdouble scale, gdouble abstol, gdouble default_val, NcmParamType ftype)
 {
   NcmSParam *default_param = ncm_sparam_new (name, symbol, lower_bound, upper_bound, scale, abstol, default_val, ftype);
   NcmVParam *vparam = ncm_vparam_new (len, default_param);
@@ -483,7 +484,7 @@ _ncm_vparam_set_property (GObject *object, guint prop_id, const GValue *value, G
 {
   NcmVParam *vparam = NCM_VPARAM (object);
   g_return_if_fail (NCM_IS_VPARAM (object));
-
+  
   switch (prop_id)
   {
 	case PROP_DEFAULT_SPARAM:
@@ -501,11 +502,13 @@ _ncm_vparam_get_property (GObject *object, guint prop_id, GValue *value, GParamS
   NcmVParam *vparam = NCM_VPARAM (object);
   g_return_if_fail (NCM_IS_VPARAM (vparam));
 
+  NCM_UNUSED (value);
+
   switch (prop_id)
   {
-	default:
-	  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-	  break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
   }
 }
 
