@@ -43,12 +43,6 @@ G_BEGIN_DECLS
 typedef struct _NcmVParamClass NcmVParamClass;
 typedef struct _NcmVParam NcmVParam;
 
-struct _NcmVParamClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-};
-
 struct _NcmVParam
 {
   /*< private >*/
@@ -58,10 +52,16 @@ struct _NcmVParam
   GPtrArray *sparam;
 };
 
+struct _NcmVParamClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+};
+
 GType ncm_vparam_get_type (void) G_GNUC_CONST;
 
 NcmVParam *ncm_vparam_new (guint len, NcmSParam *default_param);
-NcmVParam *ncm_vparam_full_new (guint len, gchar *name, gchar *symbol, gdouble lower_bound, gdouble upper_bound, gdouble scale, gdouble abstol, gdouble default_val, NcmParamType ftype);
+NcmVParam *ncm_vparam_full_new (guint len, const gchar *name, const gchar *symbol, gdouble lower_bound, gdouble upper_bound, gdouble scale, gdouble abstol, gdouble default_val, NcmParamType ftype);
 NcmVParam *ncm_vparam_copy (NcmVParam *vparam);
 void ncm_vparam_free (NcmVParam *vparam);
 void ncm_vparam_clear (NcmVParam **vparam);
