@@ -24,8 +24,8 @@
 
 /**
  * SECTION:nc_hicosmo_de
- * @title: Dark Energy Abstract Class
- * @short_description: Base class for implementing dark energy models
+ * @title: NcHICosmoDE
+ * @short_description: Abstract class for implementing dark energy models.
  *
  * FIXME
  */
@@ -103,28 +103,28 @@ static gdouble _nc_hicosmo_de_sigma_8 (NcmModel *cosmo) { return SIGMA8; }
 static gdouble _nc_hicosmo_de_powspec (NcmModel *cosmo, gdouble k) { return pow (k, SPECINDEX); }
 
 void 
-nc_hicosmo_de_set_wmap5_params (NcHICosmo *cosmo)
+nc_hicosmo_de_set_wmap5_params (NcHICosmoDE *cosmo_de)
 {
-  g_assert (NC_IS_HICOSMO_DE (cosmo));
-  ncm_model_orig_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_H0,       72.4000);
-  ncm_model_orig_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_OMEGA_C,   0.2060);
-  ncm_model_orig_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_OMEGA_X,   0.7510);
-  ncm_model_orig_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_OMEGA_B,   0.0432);
-  ncm_model_orig_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_T_GAMMA0,  2.7250);
-  ncm_model_orig_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_SPECINDEX, 0.9610);
-  ncm_model_orig_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_SIGMA8,    0.7870);
+  g_assert (NC_IS_HICOSMO_DE (cosmo_de));
+  ncm_model_orig_param_set (NCM_MODEL (cosmo_de), NC_HICOSMO_DE_H0,       72.4000);
+  ncm_model_orig_param_set (NCM_MODEL (cosmo_de), NC_HICOSMO_DE_OMEGA_C,   0.2060);
+  ncm_model_orig_param_set (NCM_MODEL (cosmo_de), NC_HICOSMO_DE_OMEGA_X,   0.7510);
+  ncm_model_orig_param_set (NCM_MODEL (cosmo_de), NC_HICOSMO_DE_OMEGA_B,   0.0432);
+  ncm_model_orig_param_set (NCM_MODEL (cosmo_de), NC_HICOSMO_DE_T_GAMMA0,  2.7250);
+  ncm_model_orig_param_set (NCM_MODEL (cosmo_de), NC_HICOSMO_DE_SPECINDEX, 0.9610);
+  ncm_model_orig_param_set (NCM_MODEL (cosmo_de), NC_HICOSMO_DE_SIGMA8,    0.7870);
 }
 
 /**
  * nc_hicosmo_de_omega_x2omega_k:
- * @cosmo: FIXME
+ * @cosmo_de: FIXME
  *
  * FIXME
  */
 void
-nc_hicosmo_de_omega_x2omega_k (NcHICosmo *cosmo)
+nc_hicosmo_de_omega_x2omega_k (NcHICosmoDE *cosmo_de)
 {
-  guint size = ncm_model_len (NCM_MODEL (cosmo));
+  guint size = ncm_model_len (NCM_MODEL (cosmo_de));
   NcmMatrix *T = ncm_matrix_new (size, size);
   NcmVector *v = ncm_vector_new (size);
   NcmReparamLinear *relin;
@@ -138,17 +138,16 @@ nc_hicosmo_de_omega_x2omega_k (NcHICosmo *cosmo)
 
   ncm_vector_set (v, NC_HICOSMO_DE_OMEGA_X, 1.0);
 
-  relin = ncm_reparam_linear_new (ncm_model_len (NCM_MODEL (cosmo)), T, v);
-  {
-    NcmReparam *reparam = NCM_REPARAM (relin);
-    NcmSParam *sp_Omegak = ncm_sparam_new ("Omegak","\\Omega_k", -5.0, 5.0, 1.0e-2, NC_HICOSMO_DEFAULT_PARAMS_ABSTOL, 0.0, NCM_PARAM_TYPE_FIXED);
-    g_ptr_array_index (reparam->sparams, NC_HICOSMO_DE_OMEGA_X) = sp_Omegak;
-  }
+  relin = ncm_reparam_linear_new (ncm_model_len (NCM_MODEL (cosmo_de)), T, v);
+  ncm_reparam_set_param_desc_full (NCM_REPARAM (relin), NC_HICOSMO_DE_OMEGA_X, 
+                                   "Omegak","\\Omega_k", -5.0, 5.0, 1.0e-2, 
+                                   NC_HICOSMO_DEFAULT_PARAMS_ABSTOL, 0.0, NCM_PARAM_TYPE_FIXED);
 
-  ncm_model_set_reparam (NCM_MODEL (cosmo), NCM_REPARAM (relin));
+  ncm_model_set_reparam (NCM_MODEL (cosmo_de), NCM_REPARAM (relin));
   
   ncm_vector_free (v);
   ncm_matrix_free (T);
+  ncm_reparam_free (NCM_REPARAM (relin));
 
   return;
 }
@@ -160,6 +159,9 @@ bbn_prior (NcmMSet *mset, gpointer obj, const gdouble *x, gdouble *f)
   gdouble z_bbn = 1.0e9;
   gdouble bbn, a;
 
+  NCM_UNUSED (obj);
+  NCM_UNUSED (x);
+  
   a = nc_hicosmo_de_weff (NC_HICOSMO_DE (cosmo), z_bbn) / nc_hicosmo_E2 (NC_HICOSMO (cosmo), z_bbn);
   bbn = 1.0 / sqrt(1.0 - a);
   f[0] = (bbn - 0.942) / 0.03;
@@ -177,7 +179,7 @@ gboolean
 nc_hicosmo_de_new_add_bbn (NcmLikelihood *lh)
 {
   NcmMSetFunc *func = ncm_mset_func_new (bbn_prior, 0, 1, NULL, NULL);
-  ncm_likelihood_priors_add (lh, func);
+  ncm_likelihood_priors_add (lh, func, FALSE);
   return TRUE;
 }
 
@@ -189,6 +191,7 @@ enum {
 static void
 nc_hicosmo_de_init (NcHICosmoDE *object)
 {
+  NCM_UNUSED (object);
 }
 
 static void
@@ -205,24 +208,23 @@ nc_hicosmo_de_class_init (NcHICosmoDEClass *klass)
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
   NcHICosmoClass* parent_class = NC_HICOSMO_CLASS (klass);
   NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
-  object_class->set_property = &ncm_model_class_set_property;
-  object_class->get_property = &ncm_model_class_get_property;
+
   object_class->finalize     = &nc_hicosmo_de_finalize;
 
   ncm_model_class_add_params (model_class, 7, 0, PROP_SIZE);
   /* Set H_0 param info */
   ncm_model_class_set_sparam (model_class, NC_HICOSMO_DE_H0, "H_0", "H0",
-                              10.0, 500.0, 1.0,
+                              10.0, 200.0, 1.0,
                               NC_HICOSMO_DEFAULT_PARAMS_ABSTOL, NC_HICOSMO_DE_DEFAULT_H0,
                               NCM_PARAM_TYPE_FIXED);
   /* Set Omega_c param info */
   ncm_model_class_set_sparam (model_class, NC_HICOSMO_DE_OMEGA_C, "\\Omega_c", "Omegac",
-                              1e-8,  10.0, 1.0e-2,
+                              1e-8,  1.2, 1.0e-2,
                               NC_HICOSMO_DEFAULT_PARAMS_ABSTOL, NC_HICOSMO_DE_DEFAULT_OMEGA_C,
                               NCM_PARAM_TYPE_FREE);
   /* Set Omega_x param info */
   ncm_model_class_set_sparam (model_class, NC_HICOSMO_DE_OMEGA_X, "\\Omega_x", "Omegax",
-                              1e-8,  10.0, 1.0e-2,
+                              1e-8,  2.0, 1.0e-2,
                               NC_HICOSMO_DEFAULT_PARAMS_ABSTOL, NC_HICOSMO_DE_DEFAULT_OMEGA_X,
                               NCM_PARAM_TYPE_FREE);
   /* Set T_gamma0 param info */
@@ -232,7 +234,7 @@ nc_hicosmo_de_class_init (NcHICosmoDEClass *klass)
                               NCM_PARAM_TYPE_FIXED);
   /* Set Omega_b param info */
   ncm_model_class_set_sparam (model_class, NC_HICOSMO_DE_OMEGA_B, "\\Omega_b", "Omegab",
-                              1e-8,  10.0, 1.0e-2,
+                              1e-8,  2.0, 1.0e-2,
                               NC_HICOSMO_DEFAULT_PARAMS_ABSTOL, NC_HICOSMO_DE_DEFAULT_OMEGA_B,
                               NCM_PARAM_TYPE_FIXED);
   /* Set n_s param info */
