@@ -24,8 +24,8 @@
 
 /**
  * SECTION:nc_distance
- * @title: Cosmological Distances and Times
- * @short_description: Calculate cosmological distances and related quantities.
+ * @title: NcDistance
+ * @short_description: Cosmological distance and time related quantities.
  *
  * This object implements several distances used in cosmology, here we have
  * the following definitions.
@@ -174,11 +174,11 @@ nc_distance_class_init (NcDistanceClass *klass)
 {
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
 
-  object_class->dispose = nc_distance_dispose;
-  object_class->finalize = nc_distance_finalize;
+  object_class->constructed  = nc_distance_constructed;
   object_class->set_property = nc_distance_set_property;
   object_class->get_property = nc_distance_get_property;
-  object_class->constructed  = nc_distance_constructed;
+  object_class->dispose = nc_distance_dispose;
+  object_class->finalize = nc_distance_finalize;
 
   g_object_class_install_property (object_class,
                                    PROP_ZF,
@@ -189,16 +189,123 @@ nc_distance_class_init (NcDistanceClass *klass)
                                                         G_MAXDOUBLE,
                                                         10.0,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+
+  klass->func_table   = g_array_new (FALSE, FALSE, sizeof (NcDistanceFunc));
+  klass->func_z_table = g_array_new (FALSE, FALSE, sizeof (NcDistanceFuncZ));
+  klass->func_hash    = g_hash_table_new (g_str_hash, g_str_equal);
+  klass->func_z_hash  = g_hash_table_new (g_str_hash, g_str_equal);
+  
+  {
+    NcDistanceFunc func_table[] = 
+    {
+      {"decoupling_redshift",              "Decoupling redshift.",              &nc_distance_decoupling_redshift,               NC_HICOSMO_IMPL_Omega_mh2},
+      {"drag_redshift",                    "Drag redshift.",                    &nc_distance_drag_redshift,                     NC_HICOSMO_IMPL_Omega_mh2},
+      {"shift_parameter_lss",              "Shift parameter at lss.",           &nc_distance_shift_parameter_lss,               NC_HICOSMO_IMPL_Omega_mh2 | NC_HICOSMO_IMPL_E2},
+      {"comoving_lss",                     "Comoving scale of lss.",            &nc_distance_comoving_lss,                      NC_HICOSMO_IMPL_Omega_mh2 | NC_HICOSMO_IMPL_E2},
+      {"acoustic_scale",                   "Acoustic scale at lss.",            &nc_distance_acoustic_scale,                    NC_HICOSMO_IMPL_Omega_mh2 | NC_HICOSMO_IMPL_E2},
+      {"angular_diameter_curvature_scale", "Angular diameter curvature scale.", &nc_distance_angular_diameter_curvature_scale,  NC_HICOSMO_IMPL_Omega_mh2 | NC_HICOSMO_IMPL_E2},
+    };
+    const guint nfuncs = sizeof (func_table) / sizeof (NcDistanceFunc);
+    guint i;
+    for (i = 0; i < nfuncs; i++)
+    {
+      g_array_append_val (klass->func_table, func_table[i]);
+      g_hash_table_insert (klass->func_hash, (gchar *)func_table[i].name, GINT_TO_POINTER (i + 1));
+    }
+  }
+
+  {
+    NcDistanceFuncZ func_z_table[] = 
+    {
+      {"d_c",      "Comoving distance.",   &nc_distance_comoving,       NC_HICOSMO_IMPL_E2},
+      {"d_t",      "Transverse distance.", &nc_distance_transverse,     NC_HICOSMO_IMPL_E2},
+      {"d_l",      "Luminozity distance.", &nc_distance_luminosity,     NC_HICOSMO_IMPL_E2},
+      {"mu",       "Distance modulus.",    &nc_distance_modulus,        NC_HICOSMO_IMPL_E2},
+      {"D_A",      "Dilation scale.",      &nc_distance_dilation_scale, NC_HICOSMO_IMPL_E2},
+      {"BAO_A",    "BAO A scale.",         &nc_distance_bao_A_scale,    NC_HICOSMO_IMPL_E2 | NC_HICOSMO_IMPL_Omega_m},
+      {"r_Dv",     "BAO r_Dv.",            &nc_distance_bao_r_Dv,       NC_HICOSMO_IMPL_E2},
+      {"sound_h",  "Sound horizon.",       &nc_distance_sound_horizon,  NC_HICOSMO_IMPL_E2 | NC_HICOSMO_IMPL_Omega_b | NC_HICOSMO_IMPL_Omega_r},
+    };
+    const guint nfuncs = sizeof (func_z_table) / sizeof (NcDistanceFuncZ);
+    guint i;
+    for (i = 0; i < nfuncs; i++)
+    {
+      g_array_append_val (klass->func_z_table, func_z_table[i]);
+      g_hash_table_insert (klass->func_z_hash, (gchar *)func_z_table[i].name, GINT_TO_POINTER (i + 1));
+    }
+  }
+}
+
+/**
+ * nc_distance_class_func_table:
+ * 
+ * Returns: (transfer container) (element-type NcDistanceFunc): the function table.
+ */
+GArray *
+nc_distance_class_func_table (void)
+{
+  NcDistanceClass *distance_class = g_type_class_ref (NC_TYPE_DISTANCE);
+  GArray *func_table = g_array_ref (distance_class->func_table);
+  g_type_class_unref (distance_class);
+  return func_table;
+}
+
+/**
+ * nc_distance_class_func_z_table:
+ * 
+ * Returns: (transfer container) (element-type NcDistanceFuncZ): the function table.
+ */
+GArray *
+nc_distance_class_func_z_table (void)
+{
+  NcDistanceClass *distance_class = g_type_class_ref (NC_TYPE_DISTANCE);
+  GArray *func_z_table = g_array_ref (distance_class->func_z_table);
+  g_type_class_unref (distance_class);
+  return func_z_table;
+}
+
+/**
+ * nc_distance_class_get_func:
+ * @name: function name.
+ * 
+ * Returns: (transfer none): the function @name or null if not found.
+ */
+NcDistanceFunc *
+nc_distance_class_get_func (const gchar *name)
+{
+  NcDistanceClass *distance_class = g_type_class_ref (NC_TYPE_DISTANCE);
+  gpointer f_i_ptr = g_hash_table_lookup (distance_class->func_hash, name);
+  guint f_i = GPOINTER_TO_INT (f_i_ptr) - 1;
+  NcDistanceFunc *f = (f_i_ptr != NULL) ? &g_array_index (distance_class->func_table, NcDistanceFunc, f_i) : NULL;
+  g_type_class_unref (distance_class);
+  return f;
+}
+
+/**
+ * nc_distance_class_get_func_z:
+ * @name: function name.
+ * 
+ * Returns: (transfer none): the function @name or null if not found.
+ */
+NcDistanceFuncZ *
+nc_distance_class_get_func_z (const gchar *name)
+{
+  NcDistanceClass *distance_class = g_type_class_ref (NC_TYPE_DISTANCE);
+  gpointer f_i_ptr = g_hash_table_lookup (distance_class->func_z_hash, name);
+  guint f_i = GPOINTER_TO_INT (f_i_ptr) - 1;
+  NcDistanceFuncZ *fz = (f_i_ptr != NULL) ? &g_array_index (distance_class->func_z_table, NcDistanceFuncZ, f_i) : NULL;
+  g_type_class_unref (distance_class);
+  return fz;
 }
 
 /**
  * nc_distance_new:
- * @z_f: final redshift $z_f$.
+ * @z_f: final redshift $z_f$
  *
  * Creates a new #NcDistance object optimized to perform distance calculations
  * to redshift up to $z_f$.
  *
- * Returns: a new #NcDistance.
+ * Returns: a new #NcDistance
  */
 NcDistance *
 nc_distance_new (gdouble z_f)
@@ -208,7 +315,7 @@ nc_distance_new (gdouble z_f)
 
 /**
  * nc_distance_ref:
- * @dist: a #NcDistance.
+ * @dist: a #NcDistance
  *
  * FIXME
  *
@@ -222,7 +329,7 @@ nc_distance_ref (NcDistance *dist)
 
 /**
  * nc_distance_free:
- * @dist: a #NcDistance.
+ * @dist: a #NcDistance
  *
  * FIXME
  *
@@ -235,7 +342,7 @@ nc_distance_free (NcDistance *dist)
 
 /**
  * nc_distance_clear:
- * @dist: a #NcDistance.
+ * @dist: a #NcDistance
  *
  * FIXME
  *
@@ -250,8 +357,8 @@ static gdouble dcddz (gdouble y, gdouble x, gpointer userdata);
 
 /**
  * nc_distance_prepare:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
  *
  * FIXME
  *
@@ -270,7 +377,8 @@ nc_distance_prepare (NcDistance *dist, NcHICosmo *cosmo)
   {
     NcmSpline *s = ncm_spline_cubic_notaknot_new ();
     dist->comoving_distance_spline =
-      ncm_ode_spline_new (s, dcddz, cosmo, 0.0, 0.0, dist->z_f);
+      ncm_ode_spline_new_full (s, dcddz, 0.0, 0.0, dist->z_f);
+    
     ncm_spline_free (s);
   }
 
@@ -283,8 +391,8 @@ nc_distance_prepare (NcDistance *dist, NcHICosmo *cosmo)
 
 /**
  * nc_distance_prepare_if_needed:
- * @dist: FIXME,
- * @cosmo: FIXME
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
  *
  * FIXME
  * 
@@ -309,13 +417,13 @@ nc_distance_init (NcDistance *dist)
 
 /**
  * nc_distance_hubble:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
  *
  * Calculate the curvature scale today as defined in Eq $\eqref{eq:def:DH}$ in
  * units of megaparsec (Mpc) [ncm_c_Mpc()].
  *
- * Returns: $D_{H0}$.
+ * Returns: $D_{H0}$
  */
 gdouble
 nc_distance_hubble (NcDistance *dist, NcHICosmo *cosmo)
@@ -328,13 +436,13 @@ static gdouble comoving_distance_integral_argument(gdouble z, gpointer p);
 
 /**
  * nc_distance_comoving:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
  * 
  * Calculate the comoving distance $D_c (z)$ as defined in Eq. $\eqref{eq:def:Dc}$.
  *
- * Returns: $D_c(z)$.
+ * Returns: $D_c(z)$
  */
 gdouble
 nc_distance_comoving (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -382,13 +490,13 @@ dcddz (gdouble cd, gdouble z, gpointer userdata)
 
 /**
  * nc_distance_transverse:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
  *
  * Compute the transverse comoving distance $D_t (z)$ defined in Eq. $\eqref{eq:def:Dt}$.
  *
- * Returns: $D_t(z)$.
+ * Returns: $D_t(z)$
  */
 gdouble
 nc_distance_transverse (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -411,29 +519,27 @@ nc_distance_transverse (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
       Dt = sinh (sqrt_Omega_k * comoving_dist) / sqrt_Omega_k;
       break;
     case 1:
-      Dt = sin (sqrt_Omega_k * comoving_dist) / sqrt_Omega_k;
+      Dt = fabs (sin (sqrt_Omega_k * comoving_dist) / sqrt_Omega_k);
       break;
     default:
       g_assert_not_reached();
       Dt = 0.0;
       break;
   }
-
-  g_assert (Dt >= 0);
   
   return Dt;
 }
 
 /**
  * nc_distance_dtransverse_dz:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
  *
  * Compute the derivative of $D_t(z)$ with respect to $z$ defined in 
  * Eq. $\eqref{eq:def:Dt}$.
  *
- * Returns: $\frac{dD_t(z)}{dz}$.
+ * Returns: $\frac{dD_t(z)}{dz}$
  */
 gdouble
 nc_distance_dtransverse_dz (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -469,13 +575,13 @@ nc_distance_dtransverse_dz (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 
 /**
  * nc_distance_luminosity:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
  *
  * Compute the luminosity distance $D_l(z)$  defined in Eq. $\eqref{eq:def:Dl}$.
  *
- * Returns: $D_l(z)$.
+ * Returns: $D_l(z)$
  */
 gdouble
 nc_distance_luminosity (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -488,13 +594,13 @@ nc_distance_luminosity (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 
 /**
  * nc_distance_modulus:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
  *
  * Compute the distance modulus $\mu(z)$ defined in Eq. $\eqref{eq:def:mu}$.
  *
- * Returns: $\mu(z)$.
+ * Returns: $\mu(z)$
  */
 gdouble
 nc_distance_modulus (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -507,14 +613,14 @@ nc_distance_modulus (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 
 /**
  * nc_distance_luminosity_hef:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z_he: redshift $z_{he}$ in our local frame.
- * @z_cmb: redshift $z_{CMB}$ in the CMB frame.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z_he: redshift $z_{he}$ in our local frame
+ * @z_cmb: redshift $z_{CMB}$ in the CMB frame
  *
  * Calculate the luminosity distance $D_l$ corrected to our local frame.
  *
- * Returns: $D_l(z_{hef},z_{CMB})$.
+ * Returns: $D_l(z_{hef},z_{CMB})$
  */
 gdouble 
 nc_distance_luminosity_hef (NcDistance *dist, NcHICosmo *cosmo, gdouble z_he, gdouble z_cmb)
@@ -527,15 +633,15 @@ nc_distance_luminosity_hef (NcDistance *dist, NcHICosmo *cosmo, gdouble z_he, gd
 
 /**
  * nc_distance_modulus_hef:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z_he: redshift $z_{he}$ in our local frame.
- * @z_cmb: redshift $z_{CMB}$ in the CMB frame.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z_he: redshift $z_{he}$ in our local frame
+ * @z_cmb: redshift $z_{CMB}$ in the CMB frame
  *
  * Calculate the distance modulus using the frame corrected luminosity distance
  * [nc_distance_luminosity_hef()].
  *
- * Returns: $\mu(z_{hef},z_{CMB})$.
+ * Returns: $\mu(z_{hef},z_{CMB})$
  */
 gdouble 
 nc_distance_modulus_hef (NcDistance *dist, NcHICosmo *cosmo, gdouble z_he, gdouble z_cmb)
@@ -557,7 +663,7 @@ nc_distance_modulus_hef (NcDistance *dist, NcHICosmo *cosmo, gdouble z_he, gdoub
  * $E(z_\star)$ is the normalized Hubble function [Eq. $\eqref{eq:def:Ez}$] and 
  * $D_t(z_\star)$ is the transverse comoving distance [Eq. $\eqref{eq:def:Dt}$] both computed at $z_\star$.
  *
- * Returns: $D_a(z_\star)$.
+ * Returns: $D_a(z_\star)$
  */
 gdouble
 nc_distance_angular_diameter_curvature_scale (NcDistance *dist, NcHICosmo *cosmo)
@@ -574,9 +680,9 @@ nc_distance_angular_diameter_curvature_scale (NcDistance *dist, NcHICosmo *cosmo
 
 /**
  * nc_distance_shift_parameter:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
  *
  * The shift parameter $R(z)$ is defined as
  * \begin{align}
@@ -587,7 +693,7 @@ nc_distance_angular_diameter_curvature_scale (NcDistance *dist, NcHICosmo *cosmo
  * $D_A(z) = D_{H_0} D_t(z) / (1 + z)$ is the angular diameter distance and 
  * $D_t(z)$ is the tranverse comoving distance [Eq. $\eqref{eq:def:Dt}$].
  *
- * Returns: $R(z)$.
+ * Returns: $R(z)$
  */
 gdouble
 nc_distance_shift_parameter (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -605,7 +711,7 @@ nc_distance_shift_parameter (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * Compute the shift parameter $R(z)$ [nc_distance_shift_parameter()] at the
  * decoupling redshift $z_\star$ [nc_distance_decoupling_redshift()].
  *
- * Returns: $R(z_\star)$.
+ * Returns: $R(z_\star)$
  */
 gdouble
 nc_distance_shift_parameter_lss (NcDistance *dist, NcHICosmo *cosmo)
@@ -623,13 +729,13 @@ nc_distance_shift_parameter_lss (NcDistance *dist, NcHICosmo *cosmo)
 
 /**
  * nc_distance_comoving_lss:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
  *
  * Compute the comoving distance $D_c(z)$ [Eq. \eqref{eq:def:Dc}] at the 
  * decoupling redshift $z_\star$ [nc_distance_decoupling_redshift()].
  *
- * Returns: $D_c(z_\star)$.
+ * Returns: $D_c(z_\star)$
  */
 gdouble
 nc_distance_comoving_lss (NcDistance *dist, NcHICosmo *cosmo)
@@ -643,8 +749,8 @@ nc_distance_comoving_lss (NcDistance *dist, NcHICosmo *cosmo)
 
 /**
  * nc_distance_decoupling_redshift:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
  *
  * The decoupling redshift $z_\star$ corresponds to the epoch of 
  * the last scattering surface of the cosmic microwave background photons. 
@@ -681,9 +787,9 @@ static gdouble sound_horizon_integral_argument(gdouble z, gpointer p);
 
 /**
  * nc_distance_sound_horizon:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
  *
  * Compute the sound horizon $r_s$, 
  * \begin{equation}
@@ -699,7 +805,7 @@ static gdouble sound_horizon_integral_argument(gdouble z, gpointer p);
  * $\Omega_b$ [nc_hicosmo_Omega_b()] and $\Omega_r$ [nc_hicosmo_Omega_r()] are the baryonic and radiation density parameter, respectively. 
  * If $\Omega_r = 0$, the integrand returns 0.0.
  *
- * Returns: $r_s(z)$.
+ * Returns: $r_s(z)$
  */
 gdouble
 nc_distance_sound_horizon (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -735,15 +841,15 @@ sound_horizon_integral_argument (gdouble z, gpointer p)
 
 /**
  * nc_distance_dsound_horizon_dz:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
  *
  * Calculate the sound horizon [nc_distance_sound_horizon()] derivative with respect to $z$, 
  * $$\frac{d r_s(z)}{dz} = - \frac{c_s(z)}{E(z)},$$
  * where $c_s(z) / E(z)$ is given by Eq. \eqref{eq:def:rs:integrand}.
  *
- * Returns: $\frac{d r_s(z)}{dz}$.
+ * Returns: $\frac{d r_s(z)}{dz}$
  */
 gdouble
 nc_distance_dsound_horizon_dz (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -754,8 +860,8 @@ nc_distance_dsound_horizon_dz (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 
 /**
  * nc_distance_acoustic_scale:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
  *
  * Compute the acoustic scale $l_A (z_\star)$ at $z_\star$ [nc_distance_decoupling_redshift()],
  * \begin{equation}
@@ -765,7 +871,7 @@ nc_distance_dsound_horizon_dz (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * and $r_s(z_\star)$ is the sound horizon [nc_distance_sound_horizon()] both 
  * both computed at $z_\star$.
  *
- * Returns: $l_A(z_\star)$.
+ * Returns: $l_A(z_\star)$
  */
 gdouble
 nc_distance_acoustic_scale (NcDistance *dist, NcHICosmo *cosmo)
@@ -779,8 +885,8 @@ nc_distance_acoustic_scale (NcDistance *dist, NcHICosmo *cosmo)
 
 /**
  * nc_distance_drag_redshift:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
  *
  * Drag redshift is the epoch at which baryons were released from photons. 
  * 
@@ -795,7 +901,7 @@ nc_distance_acoustic_scale (NcDistance *dist, NcHICosmo *cosmo)
  * $$b_1 = 0.313 (\Omega_m h^2)^{-0.419} \left(1 + 0.607 (\Omega_m h^2)^{0.674}\right) \; 
  * \text{and} \; b_2 = 0.238 (\Omega_m h^2)^{0.223}.$$
  * 
- * Returns: $z_d$.
+ * Returns: $z_d$
  */
 gdouble
 nc_distance_drag_redshift (NcDistance *dist, NcHICosmo *cosmo)
@@ -813,14 +919,22 @@ nc_distance_drag_redshift (NcDistance *dist, NcHICosmo *cosmo)
 
 /**
  * nc_distance_dilation_scale:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
  *
- * The dilation scale is the cube root of the product of the radial dilation
- * times the square of the transverse dilation -- (arXiv:astro-ph/0501171)
+ * The dilation scale is the spherically averaged distance for perturbations along and 
+ * orthogonal to the line of sight,
+ * $$D_V(z) = \left[D_{H_0}^2 D_t(z)^2 \frac{cz}{H(z)} \right]^{1/3},$$
+ * where $D_t(z)$ is the transverse comoving distance [Eq. $\eqref{eq:def:Dt}$], $c$ is the speed of light 
+ * [ncm_c_c()] and $H(z)$ is the Hubble function [nc_hicosmo_H()].  
+ * See <link linkend="XEisenstein2005">Eisenstein et al. (2005)</link>.
  *
- * Returns: $D_V(z)$.
+ * This function computes the adimensional dilation scale:
+ * $$D_V^\star(z) = \left[D_t(z)^2 \frac{z}{E(z)} \right]^{1/3} = \frac{D_V(z)}{D_{H_0}},$$
+ * where $E(z)$ is the normalized Hubble function [nc_hicosmo_E2()].
+ * 
+ * Returns: $D_V^\star(z)$
  */
 gdouble
 nc_distance_dilation_scale (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -833,13 +947,18 @@ nc_distance_dilation_scale (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 
 /**
  * nc_distance_bao_A_scale:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: the redshift $z$
  *
  * Bao 'A' scale D_v(z) sqrt(Omega_m) / z -- (arXiv:astro-ph/0501171)
+ * The acoustic scale is defined as 
+ * $$ A \equiv D_V (z) \frac{\sqrt{\Omega_m H_0^2}}{z c},$$
+ * where $\Omega_m$ is the matter density parameter [nc_hicosmo_Omega_m()], $c$ is the speed of light [ncm_c_c()], 
+ * $H_0$ is the Hubble parameter [nc_hicosmo_H0()] and $D_V(z)$ is the dilation scale [nc_distance_dilation_scale()].
+ * See Section 4.5 from <link linkend="XEisenstein2005">Eisenstein et al. (2005)</link>.
  *
- * Returns: FIXME
+ * Returns: $A(z)$
  */
 gdouble
 nc_distance_bao_A_scale (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -1002,7 +1121,7 @@ _nc_distance_free (gpointer obj)
 
 /**
  * nc_distance_func0_new:
- * @dist: FIXME
+ * @dist: a #NcDistance
  * @f0: (scope notified): FIXME
  *
  * Returns: (transfer full): FIXME
@@ -1019,11 +1138,11 @@ nc_distance_func0_new (NcDistance *dist, NcDistanceFunc0 f0)
 
 /**
  * nc_distance_func1_new:
- * @dist: FIXME
+ * @dist: a #NcDistance
  * @f1: (scope notified): FIXME
-   *
+ *
  * Returns: (transfer full): FIXME
-   */
+ */
 NcmMSetFunc *
 nc_distance_func1_new (NcDistance *dist, NcDistanceFunc1 f1)
 {
@@ -1033,3 +1152,52 @@ nc_distance_func1_new (NcDistance *dist, NcDistanceFunc1 f1)
   dist_data->f1 = f1;
   return ncm_mset_func_new (&_nc_distance_func1, 1, 1, dist_data, &_nc_distance_free);
 }
+
+typedef struct __NcDistanceArrayFunc
+{
+  guint size;
+  NcDistance *dist;
+  NcDistanceFunc1 f1;
+} _NcDistanceArrayFunc;
+
+static void
+_nc_distance_arrayfunc1_free (_NcDistanceArrayFunc *af1)
+{
+  nc_distance_clear (&af1->dist);
+  g_free (af1);
+}
+
+static void
+_nc_distance_arrayfunc1 (NcmMSet *mset, gpointer obj, const gdouble *x, gdouble *f)
+{
+  _NcDistanceArrayFunc *af1 = (_NcDistanceArrayFunc *) obj;
+  guint i;
+
+  for (i = 0; i < af1->size; i++)
+  {
+    f[i] = af1->f1 (af1->dist, NC_HICOSMO (ncm_mset_peek (mset, nc_hicosmo_id ())), x[i]);
+  }
+}
+
+/**
+ * nc_distance_create_mset_arrayfunc1:
+ * @dist: a #NcDistance
+ * @f1: (scope notified): FIXME
+ * @size: array size 
+ * 
+ * FIXME
+ * 
+ * Returns: (transfer full): FIXME
+ */
+NcmMSetFunc *
+nc_distance_create_mset_arrayfunc1 (NcDistance *dist, NcDistanceFunc1 f1, guint size)
+{
+  g_assert_cmpuint (size, !=, 0);
+  _NcDistanceArrayFunc *af1 = g_new (_NcDistanceArrayFunc, 1);
+
+  af1->size = size;
+  af1->f1 = f1;
+  af1->dist = nc_distance_ref (dist);
+
+  return ncm_mset_func_new (&_nc_distance_arrayfunc1, size, size, af1, (GDestroyNotify)_nc_distance_arrayfunc1_free);
+}
