@@ -44,42 +44,68 @@ G_BEGIN_DECLS
 
 /**
  * NcHICosmoImpl:
- * @NC_HICOSMO_IMPL_H0: FIXME
- * @NC_HICOSMO_IMPL_Omega_b: FIXME
- * @NC_HICOSMO_IMPL_Omega_r: FIXME
- * @NC_HICOSMO_IMPL_Omega_c: FIXME
- * @NC_HICOSMO_IMPL_Omega_t: FIXME
- * @NC_HICOSMO_IMPL_sigma_8: FIXME
+ * @NC_HICOSMO_IMPL_H0: Hubble constant
+ * @NC_HICOSMO_IMPL_Omega_b: Barionic density today $\Omega_b$
+ * @NC_HICOSMO_IMPL_Omega_r: Radiation density today $\Omega_r$
+ * @NC_HICOSMO_IMPL_Omega_c: Cold dark matter density today $\Omega_c$
+ * @NC_HICOSMO_IMPL_Omega_t: Total density today $\Omega_t$
+ * @NC_HICOSMO_IMPL_sigma_8: Standard deviation of the matter density contrast at scale $R = 8h^{-1} \text{Mpc}$, $\sigma_8$
  * @NC_HICOSMO_IMPL_T_gamma0: Radiation temperature today
  * @NC_HICOSMO_IMPL_z_lss: Redshift of the last scatering surface
  * @NC_HICOSMO_IMPL_as_drag: Acoustic Scale at drag redshift 
- * @NC_HICOSMO_IMPL_E2: Adimensinal Hubble function squared 
- * @NC_HICOSMO_IMPL_dE2_dz: FIXME
- * @NC_HICOSMO_IMPL_d2E2_dz2: FIXME
+ * @NC_HICOSMO_IMPL_xb: Maximum redshift 
+ * @NC_HICOSMO_IMPL_E2: Adimensional Hubble function squared 
+ * @NC_HICOSMO_IMPL_dE2_dz: Derivative of the adimensional Hubble function squared.
+ * @NC_HICOSMO_IMPL_d2E2_dz2: Second derivative of the adimensional Hubble function squared.
+ * @NC_HICOSMO_IMPL_cs2: Speed of sound squared.
+ * @NC_HICOSMO_IMPL_rhopp: energy density plus pressure.
  * @NC_HICOSMO_IMPL_cd: Comoving distance
  * @NC_HICOSMO_IMPL_powspec: Perturbations power spectrum
- *
+ * 
  * FIXME
  */
 typedef enum _NcHICosmoImpl
 {
-  NC_HICOSMO_IMPL_H0        = 1 << 0,
-  NC_HICOSMO_IMPL_Omega_b   = 1 << 1,
-  NC_HICOSMO_IMPL_Omega_r   = 1 << 2,
-  NC_HICOSMO_IMPL_Omega_c   = 1 << 3,
-  NC_HICOSMO_IMPL_Omega_t   = 1 << 4,
-  NC_HICOSMO_IMPL_sigma_8   = 1 << 5,
-  NC_HICOSMO_IMPL_T_gamma0  = 1 << 6,
-  NC_HICOSMO_IMPL_z_lss     = 1 << 7,
-  NC_HICOSMO_IMPL_as_drag   = 1 << 8,
-  NC_HICOSMO_IMPL_E2        = 1 << 9,
-  NC_HICOSMO_IMPL_dE2_dz    = 1 << 10,
-  NC_HICOSMO_IMPL_d2E2_dz2  = 1 << 11,
-  NC_HICOSMO_IMPL_cd        = 1 << 12,
-  NC_HICOSMO_IMPL_powspec   = 1 << 13, /*< private >*/
-  NC_HICOSMO_IMPL_LAST      = 1 << 14, /*< skip >*/
+  NC_HICOSMO_IMPL_H0              = 1 << 0,
+  NC_HICOSMO_IMPL_Omega_b         = 1 << 1,
+  NC_HICOSMO_IMPL_Omega_r         = 1 << 2,
+  NC_HICOSMO_IMPL_Omega_c         = 1 << 3,
+  NC_HICOSMO_IMPL_Omega_t         = 1 << 4,
+  NC_HICOSMO_IMPL_sigma_8         = 1 << 5,
+  NC_HICOSMO_IMPL_T_gamma0        = 1 << 6,
+  NC_HICOSMO_IMPL_z_lss           = 1 << 7,
+  NC_HICOSMO_IMPL_as_drag         = 1 << 8,
+  NC_HICOSMO_IMPL_xb              = 1 << 9,
+  NC_HICOSMO_IMPL_E2              = 1 << 10,
+  NC_HICOSMO_IMPL_dE2_dz          = 1 << 11,
+  NC_HICOSMO_IMPL_d2E2_dz2        = 1 << 12,
+  NC_HICOSMO_IMPL_cs2             = 1 << 13,
+  NC_HICOSMO_IMPL_rhopp           = 1 << 14,
+  NC_HICOSMO_IMPL_cd              = 1 << 15,
+  NC_HICOSMO_IMPL_powspec         = 1 << 16, /*< private >*/
+  NC_HICOSMO_IMPL_LAST            = 1 << 23, /*< skip >*/
 } NcHICosmoImpl;
 
+#define NC_HICOSMO_IMPL_c_H0 (NC_HICOSMO_IMPL_H0)
+#define NC_HICOSMO_IMPL_Omega_k (NC_HICOSMO_IMPL_Omega_t)
+#define NC_HICOSMO_IMPL_Omega_m (NC_HICOSMO_IMPL_Omega_c | NC_HICOSMO_IMPL_Omega_b)
+#define NC_HICOSMO_IMPL_h (NC_HICOSMO_IMPL_H0)
+#define NC_HICOSMO_IMPL_h2 (NC_HICOSMO_IMPL_H0)
+#define NC_HICOSMO_IMPL_Omega_bh2 (NC_HICOSMO_IMPL_Omega_b | NC_HICOSMO_IMPL_h2)
+#define NC_HICOSMO_IMPL_Omega_ch2 (NC_HICOSMO_IMPL_Omega_c | NC_HICOSMO_IMPL_h2)
+#define NC_HICOSMO_IMPL_Omega_rh2 (NC_HICOSMO_IMPL_Omega_r | NC_HICOSMO_IMPL_h2)
+#define NC_HICOSMO_IMPL_Omega_mh2 (NC_HICOSMO_IMPL_Omega_m | NC_HICOSMO_IMPL_h2)
+
+#define NC_HICOSMO_IMPL_H (NC_HICOSMO_IMPL_H0 | NC_HICOSMO_IMPL_E2)
+#define NC_HICOSMO_IMPL_dH_dz (NC_HICOSMO_IMPL_H0 | NC_HICOSMO_IMPL_E2 | NC_HICOSMO_IMPL_dE2_dz)
+#define NC_HICOSMO_IMPL_E (NC_HICOSMO_IMPL_E2)
+#define NC_HICOSMO_IMPL_Em2 (NC_HICOSMO_IMPL_E2)
+#define NC_HICOSMO_IMPL_q (NC_HICOSMO_IMPL_E2 | NC_HICOSMO_IMPL_dE2_dz)
+#define NC_HICOSMO_IMPL_j (NC_HICOSMO_IMPL_E2 | NC_HICOSMO_IMPL_dE2_dz | NC_HICOSMO_IMPL_d2E2_dz2)
+#define NC_HICOSMO_IMPL_Omega_k (NC_HICOSMO_IMPL_Omega_t)
+#define NC_HICOSMO_IMPL_wec (NC_HICOSMO_IMPL_E2 | NC_HICOSMO_IMPL_Omega_k)
+#define NC_HICOSMO_IMPL_dec (NC_HICOSMO_IMPL_E2 | NC_HICOSMO_IMPL_Omega_k)
+
 typedef struct _NcHICosmoClass NcHICosmoClass;
 typedef struct _NcHICosmo NcHICosmo;
 typedef gdouble (*NcHICosmoFunc0) (NcHICosmo *cosmo);
@@ -98,6 +124,22 @@ struct _NcHICosmo
   gboolean is_eternal;
 };
 
+typedef struct _NcHICosmoFuncZ
+{
+  const gchar *name;
+  const gchar *desc;
+  NcHICosmoFunc1 f;
+  NcHICosmoImpl impl;
+} NcHICosmoFuncZ;
+
+typedef struct _NcHICosmoFunc
+{
+  const gchar *name;
+  const gchar *desc;
+  NcHICosmoFunc0 f;
+  NcHICosmoImpl impl;
+} NcHICosmoFunc;
+
 struct _NcHICosmoClass
 {
   /*< private >*/
@@ -111,11 +153,18 @@ struct _NcHICosmoClass
   NcmModelFunc0 T_gamma0;
   NcmModelFunc0 z_lss;
   NcmModelFunc0 as_drag;
+  NcmModelFunc0 xb;
   NcmModelFunc1 E2;
   NcmModelFunc1 dE2_dz;
   NcmModelFunc1 d2E2_dz2;
+  NcmModelFunc1 cs2;
+  NcmModelFunc1 rhopp;
   NcmModelFunc1 cd;
   NcmModelFunc1 powspec;
+  GArray *func_table;
+  GArray *func_z_table;
+  GHashTable *func_hash;
+  GHashTable *func_z_hash;
 };
 
 GType nc_hicosmo_get_type (void) G_GNUC_CONST;
@@ -134,9 +183,12 @@ G_INLINE_FUNC gdouble nc_hicosmo_T_gamma0 (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_sigma_8 (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_z_lss (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_as_drag (NcHICosmo *cosmo);
+G_INLINE_FUNC gdouble nc_hicosmo_xb (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_E2 (NcHICosmo *cosmo, gdouble x);
 G_INLINE_FUNC gdouble nc_hicosmo_dE2_dz (NcHICosmo *cosmo, gdouble x);
 G_INLINE_FUNC gdouble nc_hicosmo_d2E2_dz2 (NcHICosmo *cosmo, gdouble x);
+G_INLINE_FUNC gdouble nc_hicosmo_cs2 (NcHICosmo *cosmo, gdouble x);
+G_INLINE_FUNC gdouble nc_hicosmo_rhopp (NcHICosmo *cosmo, gdouble x);
 G_INLINE_FUNC gdouble nc_hicosmo_cd (NcHICosmo *cosmo, gdouble x);
 G_INLINE_FUNC gdouble nc_hicosmo_powspec (NcHICosmo *cosmo, gdouble x);
 G_INLINE_FUNC gdouble nc_hicosmo_c_H0 (NcHICosmo *cosmo);
@@ -157,13 +209,23 @@ G_INLINE_FUNC gdouble nc_hicosmo_qp (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_q (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_dec (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_wec (NcHICosmo *cosmo, gdouble z);
+G_INLINE_FUNC gdouble nc_hicosmo_abs_alpha (NcHICosmo *cosmo, gdouble x);
+G_INLINE_FUNC gdouble nc_hicosmo_x_alpha (NcHICosmo *cosmo, gdouble alpha);
+
+GArray *nc_hicosmo_class_func_table (void);
+GArray *nc_hicosmo_class_func_z_table (void);
+NcHICosmoFunc *nc_hicosmo_class_get_func (const gchar *name);
+NcHICosmoFuncZ *nc_hicosmo_class_get_func_z (const gchar *name);
 
 NcHICosmo *nc_hicosmo_new_from_name (GType parent_type, gchar *cosmo_name);
+NcHICosmo *nc_hicosmo_ref (NcHICosmo *cosmo);
 void nc_hicosmo_log_all_models (GType parent);
-void nc_hicosmo_free (NcHICosmo *hic);
+void nc_hicosmo_free (NcHICosmo *cosmo);
+void nc_hicosmo_clear (NcHICosmo **cosmo);
 
 NcmMSetFunc *nc_hicosmo_create_mset_func0 (NcHICosmoFunc0 f0);
 NcmMSetFunc *nc_hicosmo_create_mset_func1 (NcHICosmoFunc1 f1);
+NcmMSetFunc *nc_hicosmo_create_mset_arrayfunc1 (NcHICosmoFunc1 f1, guint size);
 
 void nc_hicosmo_set_H0_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
 void nc_hicosmo_set_Omega_b_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
@@ -174,10 +236,13 @@ void nc_hicosmo_set_sigma_8_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
 void nc_hicosmo_set_T_gamma0_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
 void nc_hicosmo_set_z_lss_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
 void nc_hicosmo_set_as_drag_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
+void nc_hicosmo_set_xb_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
 
 void nc_hicosmo_set_E2_impl (NcHICosmoClass *model_class, NcmModelFunc1 f);
 void nc_hicosmo_set_dE2_dz_impl (NcHICosmoClass *model_class, NcmModelFunc1 f);
 void nc_hicosmo_set_d2E2_dz2_impl (NcHICosmoClass *model_class, NcmModelFunc1 f);
+void nc_hicosmo_set_cs2_impl (NcHICosmoClass *model_class, NcmModelFunc1 f);
+void nc_hicosmo_set_rhopp_impl (NcHICosmoClass *model_class, NcmModelFunc1 f);
 void nc_hicosmo_set_cd_impl (NcHICosmoClass *model_class, NcmModelFunc1 f);
 void nc_hicosmo_set_powspec_impl (NcHICosmoClass *model_class, NcmModelFunc1 f);
 
@@ -203,10 +268,13 @@ NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,T_gamma0)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,sigma_8)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,z_lss)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,as_drag)
+NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,xb)
 
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,E2)
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,dE2_dz)
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,d2E2_dz2)
+NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,cs2)
+NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,rhopp)
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,cd)
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,powspec)
 
@@ -343,6 +411,20 @@ nc_hicosmo_j (NcHICosmo *cosmo, gdouble z)
   return gsl_pow_2 (1.0 + z) * (d2E2_dz2 - 2.0 * dE2_dz / (1.0 + z)) / (2.0 * E2) + 1.0;
 }
 
+G_INLINE_FUNC gdouble
+nc_hicosmo_x_alpha (NcHICosmo *cosmo, gdouble alpha)
+{
+  const gdouble xb = nc_hicosmo_xb (cosmo);
+  return xb * exp (- (alpha * alpha) * 0.5);
+}
+
+G_INLINE_FUNC gdouble
+nc_hicosmo_abs_alpha (NcHICosmo *cosmo, gdouble x)
+{
+  const gdouble xb = nc_hicosmo_xb (cosmo);
+  return sqrt (2.0 * log (xb / x));
+}
+
 G_END_DECLS
 
 #endif /* NUMCOSMO_HAVE_INLINE */
