@@ -30,6 +30,7 @@
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
 #include <numcosmo/nc_hicosmo.h>
+#include <numcosmo/math/ncm_util.h>
 #include <numcosmo/math/ncm_model_ctrl.h>
 #include <numcosmo/math/ncm_spline.h>
 #include <numcosmo/math/function_cache.h>
@@ -53,32 +54,33 @@ G_BEGIN_DECLS
 typedef struct _NcRecombClass NcRecombClass;
 typedef struct _NcRecomb NcRecomb;
 
-struct _NcRecombClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-	void (*prepare) (NcRecomb *recomb, NcHICosmo *cosmo);
-};
-
 struct _NcRecomb
 {
   /*< private >*/
   GObject parent_instance;
-	gdouble zi, lambdai, lambdaf, prec;
-	gdouble init_frac;
-	gsl_min_fminimizer *fmin;
-	gsl_root_fsolver *fsol;
-	NcmSpline *Xe_s;
+  gdouble zi, lambdai, lambdaf, prec;
+  gdouble init_frac;
+  gsl_min_fminimizer *fmin;
+  gsl_root_fsolver *fsol;
+  NcmSpline *Xe_s;
   NcmSpline *dtau_dlambda_s;
-	NcmSpline *tau_s;
+  NcmSpline *tau_s;
   NcmModelCtrl *ctrl;
 };
 
+struct _NcRecombClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+  void (*prepare) (NcRecomb *recomb, NcHICosmo *cosmo);
+};
+
 GType nc_recomb_get_type (void) G_GNUC_CONST;
 
-NcRecomb *nc_recomb_new_from_name (gchar *recomb_name);
+NcRecomb *nc_recomb_new_from_name (const gchar *recomb_name);
 NcRecomb *nc_recomb_ref (NcRecomb *recomb);
 void nc_recomb_free (NcRecomb *recomb);
+void nc_recomb_clear (NcRecomb **recomb);
 void nc_recomb_prepare (NcRecomb *recomb, NcHICosmo *cosmo);
 G_INLINE_FUNC void nc_recomb_prepare_if_needed (NcRecomb *recomb, NcHICosmo *cosmo);
 
@@ -178,6 +180,7 @@ nc_recomb_pequignot_HI_case_B (NcHICosmo *cosmo, gdouble Tm)
   const gdouble d =  0.5300;
   const gdouble t = Tm * 1.0e-4;
   const gdouble res = F * G * a * pow (t, b) / (1.0 + c * pow (t, d));
+  NCM_UNUSED (cosmo);
   return res;
 }
 
@@ -194,6 +197,7 @@ nc_recomb_pequignot_HI_case_B_dTm (NcHICosmo *cosmo, gdouble Tm)
   const gdouble t_b = pow (t, b);
   const gdouble t_d = pow (t, d);
   const gdouble res = a * F * G * (b + c * (b - d) * t_d) * t_b / (Tm * gsl_pow_2 (1.0 + c * t_d));
+  NCM_UNUSED (cosmo);
   return res;
 }
 
@@ -208,6 +212,7 @@ nc_recomb_hummer_HeI_case_B (NcHICosmo *cosmo, gdouble Tm)
   const gdouble sqrt_Tm_T2 = sqrt_Tm / sqrt_T2;
   const gdouble sqrt_Tm_T1 = sqrt_Tm / sqrt_T1;
   const gdouble res = q / (sqrt_Tm_T2 * pow (1.0 + sqrt_Tm_T2, 1.0 - p) * pow (1.0 + sqrt_Tm_T1, 1.0 + p));
+  NCM_UNUSED (cosmo);
   return res;
 }
 
@@ -234,6 +239,7 @@ nc_recomb_hummer_HeI_case_B_dTm (NcHICosmo *cosmo, gdouble Tm)
       pow (1.0 + sqrt_Tm_T2, 2.0 - p) *
       pow (1.0 + sqrt_Tm_T1, 2.0 + p)
       );
+  NCM_UNUSED (cosmo);
   return res;
 }
 
